@@ -131,7 +131,7 @@ usage (void)
            "  --display=%s; --geometry=%s; --role=%s; --drop-down;\n"
            "  --startup-id=%s; -I, --icon=%s; --fullscreen; --maximize;\n"
            "  --show-menubar, --hide-menubar; --show-borders, --hide-borders;\n"
-           "  --show-toolbar, --hide-toolbar\n\n",
+           "  --show-toolbar, --hide-toolbar; --font=%s, --zoom=%s\n\n",
            _("Window Options"),
            /* parameter of --display */
            _("display"),
@@ -142,7 +142,11 @@ usage (void)
            /* parameter of --startup-id */
            _("string"),
            /* parameter of --icon */
-           _("icon"));
+           _("icon"),
+           /* parameter of --font */
+           _("font"),
+           /* parameter of --zoom */
+           _("zoom"));
 
   g_print (_("See the %s man page for full explanation of the options above."),
            PACKAGE_NAME);
@@ -192,10 +196,11 @@ main (int argc, char **argv)
   if (G_UNLIKELY (show_version))
     {
       g_print ("%s %s (Xfce %s)\n\n", PACKAGE_NAME, PACKAGE_VERSION, xfce_version_string ());
-      g_print ("%s\n", "Copyright (c) 2003-2012");
+      g_print ("%s\n", "Copyright (c) 2003-2016");
       g_print ("\t%s\n\n", _("The Xfce development team. All rights reserved."));
-      g_print ("%s\n", _("Written by Benedikt Meurer <benny@xfce.org>"));
-      g_print ("%s\n\n", _("and Nick Schermer <nick@xfce.org>."));
+      g_print ("%s\n", _("Written by Benedikt Meurer <benny@xfce.org>,"));
+      g_print ("%s\n", _("Nick Schermer <nick@xfce.org>"));
+      g_print ("%s\n\n", _("and Igor Zakharov <f2404@yandex.ru>."));
       g_print (_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
       g_print ("\n");
 
