@@ -35,7 +35,12 @@
 #include <string.h>
 #endif
 
-#include <gdk/gdkkeysyms.h>
+#include <libxfce4ui/libxfce4ui.h>
+
+#ifdef GDK_WINDOWING_X11
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#endif
 
 #include <terminal/terminal-app.h>
 #include <terminal/terminal-config.h>
@@ -65,7 +70,6 @@ static void               terminal_app_window_destroyed         (GtkWidget
                                                                  TerminalApp        *app);
 static void               terminal_app_save_yourself            (XfceSMClient       *client,
                                                                  TerminalApp        *app);
-static GdkScreen         *terminal_app_find_screen              (const gchar        *display_name);
 static void               terminal_app_open_window              (TerminalApp        *app,
                                                                  TerminalWindowAttr *attr);
 
@@ -188,11 +192,10 @@ terminal_app_update_accels (TerminalApp *app)
   g_object_get (G_OBJECT (app->preferences),
                 "shortcuts-no-menukey", &no_menukey,
                 NULL);
-
-  gtk_settings_set_string_property (gtk_settings_get_default (),
-                                    "gtk-menu-bar-accel",
-                                    no_menukey ? "" : app->initial_menu_bar_accel,
-                                    PACKAGE_NAME);
+  g_object_set (G_OBJECT (gtk_settings_get_default ()),
+                "gtk-menu-bar-accel",
+                no_menukey ? NULL : app->initial_menu_bar_accel,
+                NULL);
 }
 
 
@@ -273,7 +276,7 @@ terminal_app_accel_map_load (gpointer user_data)
       for (i = 1; i < 10; i++)
         {
           g_snprintf (name, sizeof (name), "<Actions>/terminal-window/goto-tab-%d", i);
-          gtk_accel_map_change_entry (name, GDK_0 + i, GDK_MOD1_MASK, FALSE);
+          gtk_accel_map_change_entry (name, GDK_KEY_0 + i, GDK_MOD1_MASK, FALSE);
         }
     }
 
@@ -324,7 +327,7 @@ terminal_app_create_window (TerminalApp       *app,
   if (role == NULL)
     {
       /* create a new window role */
-      new_role =  g_strdup_printf (PACKAGE_NAME "-%u-%d", (guint) time (NULL), g_random_int ());
+      new_role = g_strdup_printf ("%s-%u-%d", PACKAGE_NAME, (guint) time (NULL), g_random_int ());
       role = new_role;
     }
 
@@ -374,6 +377,9 @@ terminal_app_new_window (TerminalWindow *window,
   win_attr->display = gdk_screen_make_display_name (screen);
   tab_attr = win_attr->tabs->data;
   tab_attr->directory = g_strdup (working_directory);
+  if (window->font)
+    win_attr->font = g_strdup (window->font);
+  win_attr->zoom = window->zoom;
 
   /* check if we should try to inherit the parent geometry */
   g_object_get (G_OBJECT (app->preferences), "misc-inherit-geometry", &inherit_geometry, NULL);
@@ -477,7 +483,7 @@ terminal_app_save_yourself (XfceSMClient *client,
 
   argc = g_slist_length (result) + 1;
   argv = g_new (gchar*, argc + 1);
-  for (lp = result, n = 1; n < argc; lp = lp->next, ++n)
+  for (lp = result, n = 1; n < argc && lp != NULL; lp = lp->next, ++n)
     argv[n] = lp->data;
   argv[n] = NULL;
 
@@ -500,12 +506,12 @@ terminal_app_save_yourself (XfceSMClient *client,
 
 
 
-static GdkScreen*
-terminal_app_find_screen (const gchar *display_name)
+static GdkDisplay *
+terminal_app_find_display (const gchar *display_name,
+                           gint        *screen_num)
 {
   const gchar *other_name;
   GdkDisplay  *display = NULL;
-  GdkScreen   *screen = NULL;
   GSList      *displays;
   GSList      *dp;
   gulong       n;
@@ -517,6 +523,8 @@ terminal_app_find_screen (const gchar *display_name)
   if (display_name != NULL)
     {
       name = g_strdup (display_name);
+
+      /* extract screen number from display name */
       period = strrchr (name, '.');
       if (period != NULL)
         {
@@ -540,22 +548,39 @@ terminal_app_find_screen (const gchar *display_name)
         }
       g_slist_free (displays);
 
+      g_free (name);
+
       if (display == NULL)
         display = gdk_display_open (display_name);
+    }
 
-      if (display != NULL)
-        {
-          if (num >= 0)
-            screen = gdk_display_get_screen (display, num);
+  if (display == NULL)
+    display = gdk_display_get_default ();
 
-          if (screen == NULL)
-            screen = gdk_display_get_default_screen (display);
+  if (screen_num != NULL)
+    *screen_num = num;
 
-          if (screen != NULL)
-            g_object_ref (G_OBJECT (screen));
-        }
+  return display;
+}
 
-      g_free (name);
+
+
+static GdkScreen *
+terminal_app_find_screen (GdkDisplay *display,
+                          gint        screen_num)
+{
+  GdkScreen *screen = NULL;
+
+  if (display != NULL)
+    {
+      if (screen_num >= 0)
+        screen = gdk_display_get_screen (display, screen_num);
+
+      if (screen == NULL)
+        screen = gdk_display_get_default_screen (display);
+
+      if (screen != NULL)
+        g_object_ref (G_OBJECT (screen));
     }
 
   if (screen == NULL)
@@ -569,6 +594,18 @@ terminal_app_find_screen (const gchar *display_name)
 
 
 
+static GdkScreen *
+terminal_app_find_screen_by_name (const gchar *display_name)
+{
+  GdkDisplay *display;
+  gint        screen_num;
+
+  display = terminal_app_find_display (display_name, &screen_num);
+  return terminal_app_find_screen (display, screen_num);
+}
+
+
+
 static void
 terminal_app_open_window (TerminalApp        *app,
                           TerminalWindowAttr *attr)
@@ -580,6 +617,13 @@ terminal_app_open_window (TerminalApp        *app,
   gchar           *geometry;
   GSList          *lp;
   gboolean         reuse_window = FALSE;
+  GdkDisplay      *attr_display;
+  gint             attr_screen_num;
+#if GTK_CHECK_VERSION (3,20,0)
+  TerminalScreen  *active_terminal;
+  gint             mask = NoValue, x, y;
+  guint            width, height;
+#endif
 
   terminal_return_if_fail (TERMINAL_IS_APP (app));
   terminal_return_if_fail (attr != NULL);
@@ -587,9 +631,17 @@ terminal_app_open_window (TerminalApp        *app,
   if (attr->drop_down)
     {
       /* look for an exising drop-down window */
+      attr_display = terminal_app_find_display (attr->display, &attr_screen_num);
       for (lp = app->windows; lp != NULL; lp = lp->next)
-        if (TERMINAL_IS_WINDOW_DROPDOWN (lp->data))
-          break;
+        {
+          if (TERMINAL_IS_WINDOW_DROPDOWN (lp->data))
+            {
+              /* check if the screen of the display matches (bug #9957) */
+              screen = gtk_window_get_screen (GTK_WINDOW (lp->data));
+              if (gdk_screen_get_display (screen) == attr_display)
+                break;
+            }
+        }
 
       if (lp != NULL)
         {
@@ -614,6 +666,10 @@ terminal_app_open_window (TerminalApp        *app,
                                                   attr->fullscreen,
                                                   attr->menubar,
                                                   attr->toolbar);
+
+          /* put it on the correct screen/display */
+          screen = terminal_app_find_screen (attr_display, attr_screen_num);
+          gtk_window_set_screen (GTK_WINDOW (window), screen);
         }
     }
   else if (attr->reuse_last_window
@@ -648,7 +704,7 @@ terminal_app_open_window (TerminalApp        *app,
             gtk_window_set_icon_name (GTK_WINDOW (window), attr->icon);
         }
 
-      screen = terminal_app_find_screen (attr->display);
+      screen = terminal_app_find_screen_by_name (attr->display);
       if (G_LIKELY (screen != NULL))
         {
           gtk_window_set_screen (GTK_WINDOW (window), screen);
@@ -656,6 +712,17 @@ terminal_app_open_window (TerminalApp        *app,
         }
     }
 
+  /* font and zoom for new window */
+  if (!reuse_window)
+    {
+      if (attr->font)
+        {
+          g_free (TERMINAL_WINDOW (window)->font);
+          TERMINAL_WINDOW (window)->font = g_strdup (attr->font);
+        }
+      TERMINAL_WINDOW (window)->zoom = attr->zoom;
+    }
+
   /* add the tabs */
   for (lp = attr->tabs; lp != NULL; lp = lp->next)
     {
@@ -677,9 +744,12 @@ terminal_app_open_window (TerminalApp        *app,
 
   if (!attr->drop_down)
     {
-      /* don't apply other attributes to the window when reusing */
+      /* don't apply other attributes to the window when reusing, just present it to user */
       if (reuse_window)
-        return;
+        {
+          gtk_window_present (GTK_WINDOW (window));
+          return;
+        }
 
       /* set the window geometry, this can only be set after one of the tabs
        * has been added, because vte is the geometry widget, so atleast one
@@ -690,7 +760,23 @@ terminal_app_open_window (TerminalApp        *app,
         geometry = g_strdup (attr->geometry);
 
       /* try to apply the geometry to the window */
+#if GTK_CHECK_VERSION (3,20,0) && defined (GDK_WINDOWING_X11)
+      mask = XParseGeometry (geometry, &x, &y, &width, &height);
+      if (((mask & WidthValue) && (mask & HeightValue)) || ((mask & XValue) && (mask & YValue)))
+        {
+          if ((mask & WidthValue) && (mask & HeightValue))
+            {
+              active_terminal = terminal_window_get_active (TERMINAL_WINDOW (window));
+              if (G_LIKELY (active_terminal != NULL))
+                terminal_screen_set_size (active_terminal, width, height);
+            }
+          if ((mask & XValue) && (mask & YValue))
+            gtk_window_move (GTK_WINDOW (window), x, y);
+        }
+      else
+#else
       if (!gtk_window_parse_geometry (GTK_WINDOW (window), geometry))
+#endif
         g_printerr (_("Invalid geometry string \"%s\"\n"), geometry);
 
       /* cleanup */
@@ -730,16 +816,17 @@ terminal_app_process (TerminalApp  *app,
   if (G_UNLIKELY (attrs == NULL))
     return FALSE;
 
+  /* Connect to session manager first before starting any other windows */
   for (lp = attrs; lp != NULL; lp = lp->next)
     {
       attr = lp->data;
 
       /* take first sm client id */
-      if (sm_client_id == NULL && attr->sm_client_id != NULL)
-        sm_client_id = g_strdup (attr->sm_client_id);
-
-      terminal_app_open_window (app, attr);
-      terminal_window_attr_free (attr);
+      if (attr->sm_client_id != NULL)
+        {
+          sm_client_id = g_strdup (attr->sm_client_id);
+          break;
+        }
     }
 
   if (G_LIKELY (app->session_client == NULL))
@@ -764,6 +851,14 @@ terminal_app_process (TerminalApp  *app,
         }
     }
 
+  for (lp = attrs; lp != NULL; lp = lp->next)
+    {
+      attr = lp->data;
+
+      terminal_app_open_window (app, attr);
+      terminal_window_attr_free (attr);
+    }
+
   g_slist_free (attrs);
   g_free (sm_client_id);
 
