@@ -30,7 +30,7 @@ G_BEGIN_DECLS
 #define TERMINAL_ERROR (terminal_error_quark ())
 GQuark terminal_error_quark (void) G_GNUC_CONST;
 
-enum _TerminalError
+typedef enum
 {
   /* problem with the runtime linker */
   TERMINAL_ERROR_LINKER_FAILURE,
@@ -42,7 +42,7 @@ enum _TerminalError
   TERMINAL_ERROR_OPTIONS,
   /* general failure */
   TERMINAL_ERROR_FAILED,
-};
+} TerminalError;
 
 
 #define TERMINAL_TYPE_APP         (terminal_app_get_type ())
@@ -52,7 +52,6 @@ enum _TerminalError
 
 typedef struct _TerminalAppClass TerminalAppClass;
 typedef struct _TerminalApp      TerminalApp;
-typedef enum   _TerminalError    TerminalError;
 
 GType        terminal_app_get_type            (void) G_GNUC_CONST;
 
