@@ -24,26 +24,42 @@
 
 G_BEGIN_DECLS
 
-typedef struct _TerminalTabAttr    TerminalTabAttr;
-typedef struct _TerminalWindowAttr TerminalWindowAttr;
-typedef enum   _TerminalVisibility TerminalVisibility;
-
-enum _TerminalVisibility
+typedef enum
 {
   TERMINAL_VISIBILITY_DEFAULT,
   TERMINAL_VISIBILITY_SHOW,
   TERMINAL_VISIBILITY_HIDE
-};
+} TerminalVisibility;
 
-struct _TerminalTabAttr
+typedef enum
+{
+  TERMINAL_ZOOM_LEVEL_MINIMUM     = -7,
+  TERMINAL_ZOOM_LEVEL_XXXXX_SMALL = -6,
+  TERMINAL_ZOOM_LEVEL_XXXX_SMALL  = -5,
+  TERMINAL_ZOOM_LEVEL_XXX_SMALL   = -4,
+  TERMINAL_ZOOM_LEVEL_XX_SMALL    = -3,
+  TERMINAL_ZOOM_LEVEL_X_SMALL     = -2,
+  TERMINAL_ZOOM_LEVEL_SMALL       = -1,
+  TERMINAL_ZOOM_LEVEL_MEDIUM      = 0,
+  TERMINAL_ZOOM_LEVEL_LARGE       = +1,
+  TERMINAL_ZOOM_LEVEL_X_LARGE     = +2,
+  TERMINAL_ZOOM_LEVEL_XX_LARGE    = +3,
+  TERMINAL_ZOOM_LEVEL_XXX_LARGE   = +4,
+  TERMINAL_ZOOM_LEVEL_XXXX_LARGE  = +5,
+  TERMINAL_ZOOM_LEVEL_XXXXX_LARGE = +6,
+  TERMINAL_ZOOM_LEVEL_MAXIMUM     = +7,
+  TERMINAL_ZOOM_LEVEL_DEFAULT     = TERMINAL_ZOOM_LEVEL_MEDIUM
+} TerminalZoomLevel;
+
+typedef struct
 {
   gchar    **command;
   gchar     *directory;
   gchar     *title;
   guint      hold : 1;
-};
+} TerminalTabAttr;
 
-struct _TerminalWindowAttr
+typedef struct
 {
   GSList              *tabs;
   guint                drop_down : 1;
@@ -53,13 +69,15 @@ struct _TerminalWindowAttr
   gchar               *startup_id;
   gchar               *sm_client_id;
   gchar               *icon;
+  gchar               *font;
   guint                fullscreen : 1;
   TerminalVisibility   menubar;
   TerminalVisibility   borders;
   TerminalVisibility   toolbar;
+  TerminalZoomLevel    zoom;
   guint                maximize : 1;
   guint                reuse_last_window : 1;
-};
+} TerminalWindowAttr;
 
 void                terminal_options_parse     (gint                 argc,
                                                 gchar              **argv,
