@@ -64,6 +64,8 @@ static void terminal_preferences_dialog_background_set    (GtkFileChooserButton
                                                            TerminalPreferencesDialog *dialog);
 static void terminal_preferences_dialog_encoding_changed  (GtkComboBox               *combobox,
                                                            TerminalPreferencesDialog *dialog);
+static void terminal_preferences_dialog_scroll_unlimited  (GtkWidget                 *button,
+                                                           TerminalPreferencesDialog *dialog);
 
 
 
@@ -137,9 +139,12 @@ terminal_preferences_dialog_init (TerminalPreferencesDialog *dialog)
   const gchar      *props_active[] = { "title-mode", "command-login-shell",
                                        "command-update-records", "scrolling-single-line",
                                        "scrolling-on-output", "scrolling-on-keystroke",
-                                       "scrolling-bar", "font-allow-bold",
-                                       "misc-menubar-default", "misc-toolbar-default",
-                                       "misc-borders-default",
+                                       "scrolling-bar", "scrolling-unlimited",
+                                       "misc-cursor-shape", "misc-cursor-blinks",
+                                       "font-allow-bold", "misc-menubar-default",
+                                       "misc-toolbar-default", "misc-borders-default",
+                                       "misc-tab-close-middle-click",
+                                       "misc-mouse-autohide", "shortcuts-no-helpkey",
                                        "shortcuts-no-mnemonics", "shortcuts-no-menukey",
                                        "binding-backspace", "binding-delete",
                                        "background-mode", "background-image-style",
@@ -196,7 +201,7 @@ error:
 
   /* bind color properties */
   for (i = 0; i < G_N_ELEMENTS (props_color); i++)
-    BIND_PROPERTIES (props_color[i], "color");
+    BIND_PROPERTIES (props_color[i], "rgba");
 
   /* bind color properties */
   for (i = 0; i < G_N_ELEMENTS (props_value); i++)
@@ -223,12 +228,19 @@ error:
   /* other properties */
   BIND_PROPERTIES ("font-name", "font-name");
   BIND_PROPERTIES ("title-initial", "text");
-  BIND_PROPERTIES ("emulation", "text");
   BIND_PROPERTIES ("word-chars", "text");
   BIND_PROPERTIES ("scrolling-lines", "value");
   BIND_PROPERTIES ("tab-activity-timeout", "value");
   BIND_PROPERTIES ("background-darkness", "value");
 
+  /* unlimited scrollback button */
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "scrolling-unlimited");
+  terminal_return_if_fail (G_IS_OBJECT (object));
+  g_signal_connect (G_OBJECT (object), "realize",
+      G_CALLBACK (terminal_preferences_dialog_scroll_unlimited), dialog);
+  g_signal_connect (G_OBJECT (object), "clicked",
+      G_CALLBACK (terminal_preferences_dialog_scroll_unlimited), dialog);
+
   /* reset comparibility button */
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "reset-compatibility");
   terminal_return_if_fail (G_IS_OBJECT (object));
@@ -440,7 +452,7 @@ terminal_preferences_dialog_geometry (TerminalPreferencesDialog *dialog,
                                       gint                       rows)
 {
   gint     x, y;
-  guint    w, h;
+  guint    w = 0, h = 0;
   gchar   *geo;
   gint     mask = NoValue;
 
@@ -498,12 +510,12 @@ static void
 terminal_preferences_dialog_palette_changed (GtkWidget                 *button,
                                              TerminalPreferencesDialog *dialog)
 {
-  gchar     name[16];
-  guint     i;
-  GObject  *obj;
-  GdkColor  color;
-  gchar    *color_str;
-  GString  *array;
+  gchar    name[16];
+  guint    i;
+  GObject *obj;
+  GdkRGBA  color;
+  gchar   *color_str;
+  GString *array;
 
   array = g_string_sized_new (225);
 
@@ -513,10 +525,10 @@ terminal_preferences_dialog_palette_changed (GtkWidget                 *button,
       g_snprintf (name, sizeof (name), "color-palette%d", i);
       obj = gtk_builder_get_object (GTK_BUILDER (dialog), name);
       terminal_return_if_fail (GTK_IS_COLOR_BUTTON (obj));
-      gtk_color_button_get_color (GTK_COLOR_BUTTON (obj), &color);
+      gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (obj), &color);
 
       /* append to string */
-      color_str = gdk_color_to_string (&color);
+      color_str = gdk_rgba_to_string (&color);
       g_string_append (array, color_str);
       g_free (color_str);
 
@@ -536,12 +548,12 @@ terminal_preferences_dialog_palette_changed (GtkWidget                 *button,
 static void
 terminal_preferences_dialog_palette_notify (TerminalPreferencesDialog *dialog)
 {
-  gchar    *color_str;
-  gchar   **colors;
-  guint     i;
-  gchar     name[16];
-  GObject  *obj;
-  GdkColor  color;
+  gchar   *color_str;
+  gchar  **colors;
+  guint    i;
+  gchar    name[16];
+  GObject *obj;
+  GdkRGBA  color;
 
   g_object_get (dialog->preferences, "color-palette", &color_str, NULL);
   if (G_LIKELY (color_str != NULL))
@@ -552,14 +564,14 @@ terminal_preferences_dialog_palette_notify (TerminalPreferencesDialog *dialog)
 
       /* apply values to buttons */
       if (colors != NULL)
-        for (i = 0; colors[i] != NULL && i < 16; i++)
+        for (i = 0; i < 16 && colors[i] != NULL; i++)
           {
             g_snprintf (name, sizeof (name), "color-palette%d", i + 1);
             obj = gtk_builder_get_object (GTK_BUILDER (dialog), name);
             terminal_return_if_fail (GTK_IS_COLOR_BUTTON (obj));
 
-            if (gdk_color_parse (colors[i], &color))
-              gtk_color_button_set_color (GTK_COLOR_BUTTON (obj), &color);
+            if (gdk_rgba_parse (&color, colors[i]))
+              gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (obj), &color);
           }
 
       g_strfreev (colors);
@@ -766,7 +778,7 @@ terminal_preferences_dialog_reset_compat (GtkWidget                 *button,
 {
   GParamSpec  *spec;
   GValue       value = { 0, };
-  const gchar *properties[] = { "binding-backspace", "binding-delete", "emulation" };
+  const gchar *properties[] = { "binding-backspace", "binding-delete" };
   guint        i;
 
   for (i = 0; i < G_N_ELEMENTS (properties); i++)
@@ -896,6 +908,25 @@ terminal_preferences_dialog_encoding_changed (GtkComboBox               *combobo
 
 
 
+static void
+terminal_preferences_dialog_scroll_unlimited (GtkWidget                 *button,
+                                              TerminalPreferencesDialog *dialog)
+{
+  GObject *object;
+  gboolean unlimited;
+
+  terminal_return_if_fail (TERMINAL_IS_PREFERENCES_DIALOG (dialog));
+  terminal_return_if_fail (GTK_IS_TOGGLE_BUTTON (widget));
+
+  unlimited = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "scrolling-lines");
+  terminal_return_if_fail (G_IS_OBJECT (object));
+  g_object_set (G_OBJECT (object), "sensitive", !unlimited, NULL);
+}
+
+
+
 /**
  * terminal_preferences_dialog_new:
  *
