@@ -13,9 +13,8 @@
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  * more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -61,6 +60,17 @@ enum
   PROP_COLOR_PALETTE,
   PROP_COMMAND_UPDATE_RECORDS,
   PROP_COMMAND_LOGIN_SHELL,
+  PROP_DROPDOWN_ANIMATION_TIME,
+  PROP_DROPDOWN_KEEP_OPEN_DEFAULT,
+  PROP_DROPDOWN_KEEP_ABOVE,
+  PROP_DROPDOWN_TOGGLE_FOCUS,
+  PROP_DROPDOWN_STATUS_ICON,
+  PROP_DROPDOWN_WIDTH,
+  PROP_DROPDOWN_HEIGHT,
+  PROP_DROPDOWN_OPACITY,
+  PROP_DROPDOWN_POSITION,
+  PROP_DROPDOWN_MOVE_TO_ACTIVE,
+  PROP_DROPDOWN_ALWAYS_SHOW_TABS,
   PROP_ENCODING,
   PROP_FONT_ALLOW_BOLD,
   PROP_FONT_NAME,
@@ -80,16 +90,18 @@ enum
   PROP_MISC_TAB_CLOSE_MIDDLE_CLICK,
   PROP_MISC_TAB_POSITION,
   PROP_MISC_HIGHLIGHT_URLS,
+  PROP_MISC_MIDDLE_CLICK_OPENS_URI,
   PROP_SCROLLING_BAR,
   PROP_SCROLLING_LINES,
   PROP_SCROLLING_ON_OUTPUT,
   PROP_SCROLLING_ON_KEYSTROKE,
   PROP_SCROLLING_SINGLE_LINE,
+  PROP_SCROLLING_UNLIMITED,
+  PROP_SHORTCUTS_NO_HELPKEY,
   PROP_SHORTCUTS_NO_MENUKEY,
   PROP_SHORTCUTS_NO_MNEMONICS,
   PROP_TITLE_INITIAL,
   PROP_TITLE_MODE,
-  PROP_TERM,
   PROP_WORD_CHARS,
   PROP_TAB_ACTIVITY_COLOR,
   PROP_TAB_ACTIVITY_TIMEOUT,
@@ -142,14 +154,14 @@ static void
 transform_color_to_string (const GValue *src,
                            GValue       *dst)
 {
-  GdkColor *color;
-  gchar     buffer[32];
+  GdkRGBA *color;
+  gchar    buffer[16];
 
   color = g_value_get_boxed (src);
-  g_snprintf (buffer, 32, "#%04x%04x%04x",
-              (guint) color->red,
-              (guint) color->green,
-              (guint) color->blue);
+  g_snprintf (buffer, sizeof (buffer), "#%04x%04x%04x",
+              (guint) (color->red * 65535),
+              (guint) (color->green * 65535),
+              (guint) (color->blue * 65535));
   g_value_set_string (dst, buffer);
 }
 
@@ -168,9 +180,9 @@ static void
 transform_string_to_color (const GValue *src,
                            GValue       *dst)
 {
-  GdkColor color;
+  GdkRGBA color;
 
-  gdk_color_parse (g_value_get_string (src), &color);
+  gdk_rgba_parse (&color, g_value_get_string (src));
   g_value_set_boxed (dst, &color);
 }
 
@@ -240,12 +252,12 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
   gobject_class->set_property = terminal_preferences_set_property;
 
   /* register transform functions */
-  if (!g_value_type_transformable (GDK_TYPE_COLOR, G_TYPE_STRING))
-    g_value_register_transform_func (GDK_TYPE_COLOR, G_TYPE_STRING, transform_color_to_string);
+  if (!g_value_type_transformable (GDK_TYPE_RGBA, G_TYPE_STRING))
+    g_value_register_transform_func (GDK_TYPE_RGBA, G_TYPE_STRING, transform_color_to_string);
   if (!g_value_type_transformable (G_TYPE_STRING, G_TYPE_BOOLEAN))
     g_value_register_transform_func (G_TYPE_STRING, G_TYPE_BOOLEAN, transform_string_to_boolean);
-  if (!g_value_type_transformable (G_TYPE_STRING, GDK_TYPE_COLOR))
-    g_value_register_transform_func (G_TYPE_STRING, GDK_TYPE_COLOR, transform_string_to_color);
+  if (!g_value_type_transformable (G_TYPE_STRING, GDK_TYPE_RGBA))
+    g_value_register_transform_func (G_TYPE_STRING, GDK_TYPE_RGBA, transform_string_to_color);
   if (!g_value_type_transformable (G_TYPE_STRING, G_TYPE_DOUBLE))
     g_value_register_transform_func (G_TYPE_STRING, G_TYPE_DOUBLE, transform_string_to_double);
   if (!g_value_type_transformable (G_TYPE_STRING, G_TYPE_UINT))
@@ -270,7 +282,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_BACKGROUND_MODE] =
       g_param_spec_enum ("background-mode",
-                         "background-mode",
+                         NULL,
                          "BackgroundMode",
                          TERMINAL_TYPE_BACKGROUND,
                          TERMINAL_BACKGROUND_SOLID,
@@ -281,7 +293,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_BACKGROUND_IMAGE_FILE] =
       g_param_spec_string ("background-image-file",
-                           "background-image-file",
+                           NULL,
                            "BackgroundImageFile",
                            NULL,
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -291,7 +303,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_BACKGROUND_IMAGE_STYLE] =
       g_param_spec_enum ("background-image-style",
-                         "background-image-style",
+                         NULL,
                          "BackgroundImageStyle",
                          TERMINAL_TYPE_BACKGROUND_STYLE,
                          TERMINAL_BACKGROUND_STYLE_TILED,
@@ -307,7 +319,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_BACKGROUND_DARKNESS] =
       g_param_spec_double ("background-darkness",
-                           "background-darkness",
+                           NULL,
                            "BackgroundDarkness",
                            0.0, 1.0, 0.5,
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -317,7 +329,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_BINDING_BACKSPACE] =
       g_param_spec_enum ("binding-backspace",
-                         "binding-backspace",
+                         NULL,
                          "BindingBackspace",
                          TERMINAL_TYPE_ERASE_BINDING,
                          TERMINAL_ERASE_BINDING_AUTO,
@@ -328,7 +340,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_BINDING_DELETE] =
       g_param_spec_enum ("binding-delete",
-                         "binding-delete",
+                         NULL,
                          "BindingDelete",
                          TERMINAL_TYPE_ERASE_BINDING,
                          TERMINAL_ERASE_BINDING_AUTO,
@@ -339,7 +351,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_COLOR_FOREGROUND] =
       g_param_spec_string ("color-foreground",
-                           "color-foreground",
+                           NULL,
                            "ColorForeground",
                            "#ffffff",
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -349,7 +361,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_COLOR_BACKGROUND] =
       g_param_spec_string ("color-background",
-                           "color-background",
+                           NULL,
                            "ColorBackground",
                            "#000000",
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -359,7 +371,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_COLOR_BACKGROUND_VARY] =
       g_param_spec_boolean ("color-background-vary",
-                            "color-background-vary",
+                            NULL,
                             "ColorBackgroundVary",
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -369,7 +381,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_COLOR_CURSOR] =
       g_param_spec_string ("color-cursor",
-                           "color-cursor",
+                           NULL,
                            "ColorCursor",
                            "#00aa00",
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -379,7 +391,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_COLOR_SELECTION] =
       g_param_spec_string ("color-selection",
-                           "color-selection",
+                           NULL,
                            "ColorSelection",
                            "#ffffff",
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -389,7 +401,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_COLOR_SELECTION_USE_DEFAULT] =
       g_param_spec_boolean ("color-selection-use-default",
-                            "color-selection-use-default",
+                            NULL,
                             "ColorSelectionUseDefault",
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -399,7 +411,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_COLOR_BOLD] =
       g_param_spec_string ("color-bold",
-                           "color-bold",
+                           NULL,
                            "ColorBold",
                            "#ffffff",
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -409,7 +421,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_COLOR_BOLD_USE_DEFAULT] =
       g_param_spec_boolean ("color-bold-use-default",
-                            "color-bold-use-default",
+                            NULL,
                             "ColorBoldUseDefault",
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -419,7 +431,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_COLOR_PALETTE] =
       g_param_spec_string ("color-palette",
-                           "color-palette",
+                           NULL,
                            "ColorPalette",
                            "#000000;"
                            "#aa0000;"
@@ -444,7 +456,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_TAB_ACTIVITY_COLOR] =
       g_param_spec_string ("tab-activity-color",
-                           "tab-activity-color",
+                           NULL,
                            "TabActivityColor",
                            "#aa0000",
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -454,7 +466,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_TAB_ACTIVITY_TIMEOUT] =
       g_param_spec_uint ("tab-activity-timeout",
-                         "tab-activity-timeout",
+                         NULL,
                          "TabActivityTimeout",
                          0, 30, 2,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -464,7 +476,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_COMMAND_UPDATE_RECORDS] =
       g_param_spec_boolean ("command-update-records",
-                            "command-update-records",
+                            NULL,
                             "CommandUpdateRecords",
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -474,17 +486,130 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_COMMAND_LOGIN_SHELL] =
       g_param_spec_boolean ("command-login-shell",
-                            "command-login-shell",
+                            NULL,
                             "CommandLoginShell",
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:dropdown-animation-time:
+   **/
+  preferences_props[PROP_DROPDOWN_ANIMATION_TIME] =
+      g_param_spec_uint ("dropdown-animation-time",
+                         NULL,
+                         "DropdownAnimationTime",
+                         0, 500, 0,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * TerminalPreferences:dropdown-keep-open-default:
+   **/
+  preferences_props[PROP_DROPDOWN_KEEP_OPEN_DEFAULT] =
+      g_param_spec_boolean ("dropdown-keep-open-default",
+                            NULL,
+                            "DropdownKeepOpenDefault",
+                            FALSE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+
+  /**
+   * TerminalPreferences:dropdown-keep-above:
+   **/
+  preferences_props[PROP_DROPDOWN_KEEP_ABOVE] =
+      g_param_spec_boolean ("dropdown-keep-above",
+                            NULL,
+                            "DropdownKeepAbove",
+                            TRUE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+
+  /**
+   * TerminalPreferences:dropdown-toggle-focus:
+   **/
+  preferences_props[PROP_DROPDOWN_TOGGLE_FOCUS] =
+      g_param_spec_boolean ("dropdown-toggle-focus",
+                            NULL,
+                            "DropdownToggleFocus",
+                            TRUE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * TerminalPreferences:dropdown-status-icon:
+   **/
+  preferences_props[PROP_DROPDOWN_STATUS_ICON] =
+      g_param_spec_boolean ("dropdown-status-icon",
+                            NULL,
+                            "DropdownStatusIcon",
+                            TRUE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+
+  /**
+   * TerminalPreferences:dropdown-width:
+   **/
+  preferences_props[PROP_DROPDOWN_WIDTH] =
+      g_param_spec_uint ("dropdown-width",
+                         NULL,
+                         "DropdownWidth",
+                         10, 100, 80,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * TerminalPreferences:dropdown-height:
+   **/
+  preferences_props[PROP_DROPDOWN_HEIGHT] =
+      g_param_spec_uint ("dropdown-height",
+                         NULL,
+                         "DropdownHeight",
+                         10, 100, 50,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * TerminalPreferences:dropdown-opacity:
+   **/
+  preferences_props[PROP_DROPDOWN_OPACITY] =
+      g_param_spec_uint ("dropdown-opacity",
+                         NULL,
+                         "DropdownOpacity",
+                         0, 100, 100,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * TerminalPreferences:dropdown-position:
+   **/
+  preferences_props[PROP_DROPDOWN_POSITION] =
+      g_param_spec_uint ("dropdown-position",
+                         NULL,
+                         "DropdownPosition",
+                         0, 100, 50,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * TerminalPreferences:move-to-active:
+   **/
+  preferences_props[PROP_DROPDOWN_MOVE_TO_ACTIVE] =
+      g_param_spec_boolean ("dropdown-move-to-active",
+                            NULL,
+                            "DropdownMoveToActive",
+                            TRUE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * TerminalPreferences:dropdown-always-show-tabs:
+   **/
+  preferences_props[PROP_DROPDOWN_ALWAYS_SHOW_TABS] =
+      g_param_spec_boolean ("dropdown-always-show-tabs",
+                            NULL,
+                            "DropdownAlwaysShowTabs",
+                            TRUE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:encoding:
    **/
   preferences_props[PROP_ENCODING] =
       g_param_spec_string ("encoding",
-                           "encodinge",
+                           NULL,
                            "Encoding",
                            NULL,
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -494,7 +619,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_FONT_ALLOW_BOLD] =
       g_param_spec_boolean ("font-allow-bold",
-                            "font-allow-bold",
+                            NULL,
                             "FontAllowBold",
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -504,7 +629,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_FONT_NAME] =
       g_param_spec_string ("font-name",
-                           "font-name",
+                           NULL,
                            "FontName",
                            "Monospace 12",
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -514,7 +639,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_ALWAYS_SHOW_TABS] =
       g_param_spec_boolean ("misc-always-show-tabs",
-                            "misc-always-show-tabs",
+                            NULL,
                             "MiscAlwaysShowTabs",
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -524,7 +649,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_BELL] =
       g_param_spec_boolean ("misc-bell",
-                            "misc-bell",
+                            NULL,
                             "MiscBell",
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -534,7 +659,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_BORDERS_DEFAULT] =
       g_param_spec_boolean ("misc-borders-default",
-                            "misc-borders-default",
+                            NULL,
                             "MiscBordersDefault",
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -544,7 +669,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_CURSOR_BLINKS] =
       g_param_spec_boolean ("misc-cursor-blinks",
-                            "misc-cursor-blinks",
+                            NULL,
                             "MiscCursorBlinks",
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -554,7 +679,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_CURSOR_SHAPE] =
       g_param_spec_enum ("misc-cursor-shape",
-                         "misc-cursor-shape",
+                         NULL,
                          "MiscCursorShape",
                          TERMINAL_TYPE_CURSOR_SHAPE,
                          TERMINAL_CURSOR_SHAPE_BLOCK,
@@ -565,7 +690,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_DEFAULT_GEOMETRY] =
       g_param_spec_string ("misc-default-geometry",
-                           "misc-default-geometry",
+                           NULL,
                            "MiscDefaultGeometry",
                            "80x24",
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -575,7 +700,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_INHERIT_GEOMETRY] =
       g_param_spec_boolean ("misc-inherit-geometry",
-                            "misc-inherit-geometry",
+                            NULL,
                             "MiscInheritGeometry",
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -585,7 +710,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_MENUBAR_DEFAULT] =
       g_param_spec_boolean ("misc-menubar-default",
-                            "misc-menubar-default",
+                            NULL,
                             "MiscMenubarDefault",
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -595,7 +720,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_MOUSE_AUTOHIDE] =
       g_param_spec_boolean ("misc-mouse-autohide",
-                            "misc-mouse-autohide",
+                            NULL,
                             "MiscMouseAutohide",
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -605,7 +730,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_TOOLBAR_DEFAULT] =
       g_param_spec_boolean ("misc-toolbar-default",
-                            "misc-toolbar-default",
+                            NULL,
                             "MiscToolbarDefault",
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -615,7 +740,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_CONFIRM_CLOSE] =
       g_param_spec_boolean ("misc-confirm-close",
-                            "misc-confirm-close",
+                            NULL,
                             "MiscConfirmClose",
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -625,7 +750,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_CYCLE_TABS] =
       g_param_spec_boolean ("misc-cycle-tabs",
-                            "misc-cycle-tabs",
+                            NULL,
                             "MiscCycleTabs",
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -635,7 +760,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_TAB_CLOSE_BUTTONS] =
       g_param_spec_boolean ("misc-tab-close-buttons",
-                            "misc-tab-close-buttons",
+                            NULL,
                             "MiscTabCloseButtons",
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -645,7 +770,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_TAB_CLOSE_MIDDLE_CLICK] =
       g_param_spec_boolean ("misc-tab-close-middle-click",
-                            "misc-tab-close-middle-click",
+                            NULL,
                             "MiscTabCloseMiddleClick",
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -655,7 +780,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_TAB_POSITION] =
       g_param_spec_enum ("misc-tab-position",
-                         "misc-tab-position",
+                         NULL,
                          "MiscTabPosition",
                          GTK_TYPE_POSITION_TYPE,
                          GTK_POS_TOP,
@@ -666,17 +791,27 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_MISC_HIGHLIGHT_URLS] =
       g_param_spec_boolean ("misc-highlight-urls",
-                            "misc-highlight-urls",
+                            NULL,
                             "MiscHighlightUrls",
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:misc-middle-click-open-uri:
+   **/
+  preferences_props[PROP_MISC_MIDDLE_CLICK_OPENS_URI] =
+      g_param_spec_boolean ("misc-middle-click-opens-uri",
+                            NULL,
+                            "MiscMiddleClickOpensUri",
+                            FALSE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:scrolling-bar:
    **/
   preferences_props[PROP_SCROLLING_BAR] =
       g_param_spec_enum ("scrolling-bar",
-                         "scrolling-bar",
+                         NULL,
                          "ScrollingBar",
                          TERMINAL_TYPE_SCROLLBAR,
                          TERMINAL_SCROLLBAR_RIGHT,
@@ -687,17 +822,27 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_SCROLLING_LINES] =
       g_param_spec_uint ("scrolling-lines",
-                         "scrolling-lines",
+                         NULL,
                          "ScrollingLines",
                          0u, 1024u * 1024u, 1000u,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:scrolling-unlimited:
+   **/
+  preferences_props[PROP_SCROLLING_UNLIMITED] =
+      g_param_spec_boolean ("scrolling-unlimited",
+                            NULL,
+                            "ScrollingUnlimited",
+                            FALSE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:scrolling-on-output:
    **/
   preferences_props[PROP_SCROLLING_ON_OUTPUT] =
       g_param_spec_boolean ("scrolling-on-output",
-                            "scrolling-on-output",
+                            NULL,
                             "ScrollingOnOutput",
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -707,7 +852,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_SCROLLING_ON_KEYSTROKE] =
       g_param_spec_boolean ("scrolling-on-keystroke",
-                            "scrolling-on-keystroke",
+                            NULL,
                             "ScrollingOnKeystroke",
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -719,11 +864,23 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_SCROLLING_SINGLE_LINE] =
       g_param_spec_boolean ("scrolling-single-line",
-                            "scrolling-single-line",
+                            NULL,
                             "ScrollingSingleLine",
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:shortcuts-no-helpkey:
+   *
+   * Disable help shortcut key (F1).
+   **/
+  preferences_props[PROP_SHORTCUTS_NO_HELPKEY] =
+      g_param_spec_boolean ("shortcuts-no-helpkey",
+                            NULL,
+                            "ShortcutsNoHelpkey",
+                            FALSE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:shortcuts-no-menukey:
    *
@@ -731,7 +888,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_SHORTCUTS_NO_MENUKEY] =
       g_param_spec_boolean ("shortcuts-no-menukey",
-                            "shortcuts-no-menukey",
+                            NULL,
                             "ShortcutsNoMenukey",
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -741,7 +898,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_SHORTCUTS_NO_MNEMONICS] =
       g_param_spec_boolean ("shortcuts-no-mnemonics",
-                            "shortcuts-no-mnemonics",
+                            NULL,
                             "ShortcutsNoMnemonics",
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -751,7 +908,7 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_TITLE_INITIAL] =
       g_param_spec_string ("title-initial",
-                           "title-initial",
+                           NULL,
                            "TitleInitial",
                            _("Terminal"),
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -761,28 +918,18 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
    **/
   preferences_props[PROP_TITLE_MODE] =
       g_param_spec_enum ("title-mode",
-                         "title-mode",
+                         NULL,
                          "TitleMode",
                          TERMINAL_TYPE_TITLE,
                          TERMINAL_TITLE_APPEND,
                          G_PARAM_READWRITE| G_PARAM_STATIC_STRINGS);
 
-  /**
-   * TerminalPreferences:term:
-   **/
-  preferences_props[PROP_TERM] =
-      g_param_spec_string ("term",
-                           "term",
-                           "Term",
-                           "xterm",
-                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-
   /**
    * TerminalPreferences:word-chars:
    **/
   preferences_props[PROP_WORD_CHARS] =
       g_param_spec_string ("word-chars",
-                           "word-chars",
+                           NULL,
                            "WordChars",
                            "-A-Za-z0-9,./?%&#:_=+@~",
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
@@ -984,6 +1131,7 @@ terminal_preferences_load (TerminalPreferences *preferences)
       name = g_param_spec_get_name (pspec);
 
 #ifdef G_ENABLE_DEBUG
+      terminal_assert (g_param_spec_get_nick (pspec) != NULL);
       terminal_preferences_check_blurb (pspec);
 #endif
 
@@ -1349,7 +1497,7 @@ terminal_preferences_get (void)
 gboolean
 terminal_preferences_get_color (TerminalPreferences *preferences,
                                 const gchar         *property,
-                                GdkColor            *color_return)
+                                GdkRGBA             *color_return)
 {
   gchar    *spec;
   gboolean  succeed = FALSE;
@@ -1358,7 +1506,7 @@ terminal_preferences_get_color (TerminalPreferences *preferences,
 
   g_object_get (G_OBJECT (preferences), property, &spec, NULL);
   if (G_LIKELY (spec != NULL))
-    succeed = gdk_color_parse (spec, color_return);
+    succeed = gdk_rgba_parse (color_return, spec);
   g_free (spec);
 
   return succeed;
