@@ -84,9 +84,6 @@ void         terminal_screen_select_all                   (TerminalScreen *scree
 void         terminal_screen_reset                        (TerminalScreen *screen,
                                                            gboolean        clear);
 
-void         terminal_screen_im_append_menuitems          (TerminalScreen *screen,
-                                                           GtkMenuShell   *menushell);
-
 GSList      *terminal_screen_get_restart_command          (TerminalScreen *screen);
 
 void         terminal_screen_reset_activity               (TerminalScreen *screen);
@@ -110,6 +107,7 @@ void         terminal_screen_search_find_next             (TerminalScreen *scree
 
 void         terminal_screen_search_find_previous         (TerminalScreen *screen);
 
+void         terminal_screen_update_font                  (TerminalScreen *screen);
 
 G_END_DECLS
 
