@@ -83,6 +83,7 @@ terminal_search_dialog_class_init (TerminalSearchDialogClass *klass)
 static void
 terminal_search_dialog_init (TerminalSearchDialog *dialog)
 {
+  GtkWidget *close;
   GtkWidget *hbox;
   GtkWidget *vbox;
   GtkWidget *label;
@@ -90,64 +91,56 @@ terminal_search_dialog_init (TerminalSearchDialog *dialog)
   gtk_window_set_title (GTK_WINDOW (dialog), _("Find"));
   gtk_window_set_default_size (GTK_WINDOW (dialog), 400, -1);
 
-  gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+  close = xfce_gtk_button_new_mixed ("window-close", _("_Close"));
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), close, GTK_RESPONSE_CLOSE);
+  gtk_widget_set_can_default (close, TRUE);
 
-  dialog->button_prev = xfce_gtk_button_new_mixed (GTK_STOCK_GO_BACK, _("_Previous"));
+  dialog->button_prev = xfce_gtk_button_new_mixed ("go-previous", _("_Previous"));
   gtk_dialog_add_action_widget (GTK_DIALOG (dialog), dialog->button_prev, TERMINAL_RESPONSE_SEARCH_PREV);
   gtk_widget_set_can_default (dialog->button_prev, TRUE);
-  gtk_widget_show (dialog->button_prev);
 
-  dialog->button_next = xfce_gtk_button_new_mixed (GTK_STOCK_GO_FORWARD, _("_Next"));
+  dialog->button_next = xfce_gtk_button_new_mixed ("go-next", _("_Next"));
   gtk_dialog_add_action_widget (GTK_DIALOG (dialog), dialog->button_next, TERMINAL_RESPONSE_SEARCH_NEXT);
-  gtk_widget_show (dialog->button_next);
 
-  vbox = gtk_vbox_new (FALSE, 6);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), vbox, TRUE, TRUE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
-  gtk_widget_show (vbox);
 
-  hbox = gtk_hbox_new (FALSE, 12);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
-  gtk_widget_show (hbox);
 
   label = gtk_label_new_with_mnemonic (_("_Search for:"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
 
   dialog->entry = gtk_entry_new ();
   gtk_box_pack_start (GTK_BOX (hbox), dialog->entry, TRUE, TRUE, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), dialog->entry);
   gtk_entry_set_activates_default (GTK_ENTRY (dialog->entry), TRUE);
-  gtk_entry_set_icon_from_stock (GTK_ENTRY (dialog->entry), GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_CLEAR);
+  gtk_entry_set_icon_from_icon_name (GTK_ENTRY (dialog->entry), GTK_ENTRY_ICON_SECONDARY, "edit-clear");
   g_signal_connect (G_OBJECT (dialog->entry), "icon-release",
       G_CALLBACK (terminal_search_dialog_entry_icon_release), NULL);
   g_signal_connect (G_OBJECT (dialog->entry), "changed",
       G_CALLBACK (terminal_search_dialog_entry_changed), dialog);
-  gtk_widget_show (dialog->entry);
 
   dialog->match_case = gtk_check_button_new_with_mnemonic (_("C_ase sensitive"));
   gtk_box_pack_start (GTK_BOX (vbox), dialog->match_case, FALSE, FALSE, 0);
   g_signal_connect_swapped (G_OBJECT (dialog->match_case), "toggled",
       G_CALLBACK (terminal_search_dialog_clear_gregex), dialog);
-  gtk_widget_show (dialog->match_case);
 
   dialog->match_regex = gtk_check_button_new_with_mnemonic (_("Match as _regular expression"));
   gtk_box_pack_start (GTK_BOX (vbox), dialog->match_regex, FALSE, FALSE, 0);
   g_signal_connect_swapped (G_OBJECT (dialog->match_regex), "toggled",
       G_CALLBACK (terminal_search_dialog_clear_gregex), dialog);
-  gtk_widget_show (dialog->match_regex);
 
   dialog->match_word = gtk_check_button_new_with_mnemonic (_("Match _entire word only"));
   gtk_box_pack_start (GTK_BOX (vbox), dialog->match_word, FALSE, FALSE, 0);
   g_signal_connect_swapped (G_OBJECT (dialog->match_word), "toggled",
       G_CALLBACK (terminal_search_dialog_clear_gregex), dialog);
-  gtk_widget_show (dialog->match_word);
 
   dialog->wrap_around = gtk_check_button_new_with_mnemonic (_("_Wrap around"));
   gtk_box_pack_start (GTK_BOX (vbox), dialog->wrap_around, FALSE, FALSE, 0);
   g_signal_connect_swapped (G_OBJECT (dialog->wrap_around), "toggled",
       G_CALLBACK (terminal_search_dialog_clear_gregex), dialog);
-  gtk_widget_show (dialog->wrap_around);
 
   terminal_search_dialog_entry_changed (dialog->entry, dialog);
 }
@@ -283,6 +276,7 @@ terminal_search_dialog_present (TerminalSearchDialog  *dialog)
 {
   terminal_return_if_fail (TERMINAL_IS_SEARCH_DIALOG (dialog));
 
+  gtk_widget_show_all (GTK_WIDGET (dialog));
   gtk_window_present (GTK_WINDOW (dialog));
   gtk_widget_grab_focus (dialog->entry);
 }
