@@ -28,7 +28,6 @@
 #include <string.h>
 #endif
 
-#include <gdk/gdkkeysyms.h>
 #include <libxfce4ui/libxfce4ui.h>
 
 #include <terminal/terminal-util.h>
@@ -40,17 +39,20 @@
 
 
 
-#define USERCHARS       "-_+[:alnum:]"
+#define MAILTO          "mailto:"
+#define USERCHARS       "-[:alnum:]\\Q_.+\\E"
 #define USERCHARS_CLASS "[" USERCHARS "]"
 #define PASSCHARS_CLASS "[-[:alnum:]\\Q,?;.:/!%$^*&~\"#'\\E]"
 #define HOSTCHARS_CLASS "[-[:alnum:]]"
-#define HOST            HOSTCHARS_CLASS "+(\\." HOSTCHARS_CLASS "+)*"
+#define HOSTNAME        HOSTCHARS_CLASS "+(?:\\." HOSTCHARS_CLASS "+)*"
+#define IPV6ADDRESS     "\\[(?:[[:xdigit:]]{0,4}:){2,7}[[:xdigit:]]{0,4}\\]"
+#define HOST            "(?:" HOSTNAME "|" IPV6ADDRESS ")"
 #define PORT            "(?:\\:[[:digit:]]{1,5})?"
 #define PATHCHARS_CLASS "[-[:alnum:]\\Q_$.+!*,;@&=?/:~#'%\\E]"
-#define PATHTERM_CLASS  "[^\\Q]'.}>) \t\r\n,\"\\E]"
-#define SCHEME          "(?:news:|telnet:|nntp:|file:\\/|https?:|ftps?:|sftp:|webcal:)"
+#define PATHTERM_CLASS  "[[:alnum:]\\Q_$+*@&=/~#%\\E]"
+#define SCHEME          "(?:news:|telnet:|nntp:|file:\\/|https?:|ftps?:|sftp:|webcal:|magnet:)"
 #define USERPASS        USERCHARS_CLASS "+(?:" PASSCHARS_CLASS "+)?"
-#define URLPATH         "(?:(/"PATHCHARS_CLASS"+(?:[(]"PATHCHARS_CLASS"*[)])*"PATHCHARS_CLASS"*)*"PATHTERM_CLASS")?"
+#define URLPATH         "(?:(?:\\(" PATHCHARS_CLASS "*\\)|" PATHCHARS_CLASS ")*(?:\\(" PATHCHARS_CLASS "*\\)|" PATHTERM_CLASS "))?"
 
 
 
@@ -60,27 +62,27 @@ enum
   LAST_SIGNAL,
 };
 
-enum
+typedef enum
 {
   PATTERN_TYPE_NONE,
   PATTERN_TYPE_FULL_HTTP,
   PATTERN_TYPE_HTTP,
   PATTERN_TYPE_EMAIL
-};
+} PatternType;
 
 typedef struct
 {
   const gchar *pattern;
-  gint         type;
-}
-TerminalRegexPattern;
+  PatternType  type;
+} TerminalRegexPattern;
 
 static const TerminalRegexPattern regex_patterns[] =
 {
   { SCHEME "//(?:" USERPASS "\\@)?" HOST PORT URLPATH, PATTERN_TYPE_FULL_HTTP },
-  { "(?:www|ftp)" HOSTCHARS_CLASS "*\\." HOST PORT URLPATH, PATTERN_TYPE_HTTP },
-  { "(?:mailto:)?" USERCHARS_CLASS "[" USERCHARS ".]*\\@" HOSTCHARS_CLASS "+\\." HOST, PATTERN_TYPE_EMAIL },
-  { "news:[[:alnum:]\\Q^_{|}~!\"#$%&'()*+,./;:=?`\\E]+", PATTERN_TYPE_FULL_HTTP }
+  { "(?:www[[:digit:]]{0,3}|ftp)" HOSTCHARS_CLASS "*\\." HOST PORT URLPATH, PATTERN_TYPE_HTTP },
+  { "(?:" MAILTO ")?" USERCHARS_CLASS "[" USERCHARS ".]*\\@" HOSTCHARS_CLASS "+\\." HOST, PATTERN_TYPE_EMAIL },
+  { "news:[[:alnum:]\\Q^_{|}~!\"#$%&'()*+,./;:=?`\\E]+", PATTERN_TYPE_FULL_HTTP },
+  { "magnet:[-[:alnum:]\\Q^_{|}~!\"#$%&'()*+,./;:=?`\\E]+", PATTERN_TYPE_FULL_HTTP }
 };
 
 
@@ -232,9 +234,9 @@ terminal_widget_context_menu_copy (TerminalWidget *widget,
       display = gtk_widget_get_display (GTK_WIDGET (widget));
 
       /* strip mailto from links, bug #7909 */
-      if (g_str_has_prefix (wlink, "mailto:"))
+      if (g_str_has_prefix (wlink, MAILTO))
         {
-          modified_wlink = g_strdup (wlink + 7);
+          modified_wlink = g_strdup (wlink + strlen (MAILTO));
           wlink = modified_wlink;
         }
 
@@ -272,8 +274,7 @@ static void
 terminal_widget_context_menu (TerminalWidget *widget,
                               gint            button,
                               guint32         event_time,
-                              gint            x,
-                              gint            y)
+                              GdkEvent       *event)
 {
   VteTerminal *terminal = VTE_TERMINAL (widget);
   GMainLoop   *loop;
@@ -285,14 +286,14 @@ terminal_widget_context_menu (TerminalWidget *widget,
   gchar       *match;
   guint        id, i;
   gint         tag;
-  gint         pattern_type = PATTERN_TYPE_NONE;
+  PatternType  pattern_type = PATTERN_TYPE_NONE;
 
   g_signal_emit (G_OBJECT (widget), widget_signals[GET_CONTEXT_MENU], 0, &menu);
   if (G_UNLIKELY (menu == NULL))
     return;
 
   /* check if we have a match */
-  match = vte_terminal_match_check (terminal, x / terminal->char_width, y / terminal->char_height, &tag);
+  match = vte_terminal_match_check_event (terminal, event, &tag);
   if (G_UNLIKELY (match != NULL))
     {
       /* prepend a separator to the menu if it does not already contain one */
@@ -302,7 +303,6 @@ terminal_widget_context_menu (TerminalWidget *widget,
         {
           item_separator = gtk_separator_menu_item_new ();
           gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item_separator);
-          gtk_widget_show (item_separator);
         }
       else
         item_separator = NULL;
@@ -333,18 +333,18 @@ terminal_widget_context_menu (TerminalWidget *widget,
       g_object_set_data_full (G_OBJECT (item_copy), I_("terminal-widget-link"), g_strdup (match), g_free);
       g_signal_connect_swapped (G_OBJECT (item_copy), "activate", G_CALLBACK (terminal_widget_context_menu_copy), widget);
       gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item_copy);
-      gtk_widget_show (item_copy);
 
       /* prepend the "OPEN" menu item */
       g_object_set_data_full (G_OBJECT (item_open), I_("terminal-widget-link"), g_strdup (match), g_free);
       g_object_set_data_full (G_OBJECT (item_open), I_("terminal-widget-tag"), g_memdup (&tag, sizeof (tag)), g_free);
       g_signal_connect_swapped (G_OBJECT (item_open), "activate", G_CALLBACK (terminal_widget_context_menu_open), widget);
       gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item_open);
-      gtk_widget_show (item_open);
 
       g_free (match);
     }
 
+  gtk_widget_show_all (menu);
+
   /* take a reference on the menu */
   g_object_ref_sink (G_OBJECT (menu));
 
@@ -396,28 +396,33 @@ terminal_widget_button_press_event (GtkWidget       *widget,
                                     GdkEventButton  *event)
 {
   gboolean committed = FALSE;
+  gboolean middle_click_opens_uri;
   gchar   *match;
   guint    signal_id = 0;
   gint     tag;
 
-  if (event->button == 2 && event->type == GDK_BUTTON_PRESS)
+  if (event->type == GDK_BUTTON_PRESS)
     {
-      /* middle-clicking on an URI fires the responsible application */
-      match = vte_terminal_match_check (VTE_TERMINAL (widget),
-                                        event->x / VTE_TERMINAL (widget)->char_width,
-                                        event->y / VTE_TERMINAL (widget)->char_height,
-                                        &tag);
-      if (G_UNLIKELY (match != NULL))
+      /* check whether to use ctrl-click or middle click to open URI */
+      g_object_get (G_OBJECT (TERMINAL_WIDGET (widget)->preferences),
+          "misc-middle-click-opens-uri", &middle_click_opens_uri, NULL);
+
+      if (middle_click_opens_uri ? (event->button == 2) : (event->button == 1 && event->state == GDK_CONTROL_MASK))
         {
-          terminal_widget_open_uri (TERMINAL_WIDGET (widget), match, tag);
-          g_free (match);
-          return TRUE;
+          /* clicking on an URI fires the responsible application */
+          match = vte_terminal_match_check_event (VTE_TERMINAL (widget), (GdkEvent *) event, &tag);
+          if (G_UNLIKELY (match != NULL))
+            {
+              terminal_widget_open_uri (TERMINAL_WIDGET (widget), match, tag);
+              g_free (match);
+              return TRUE;
+            }
+        }
+      else if (event->button == 3)
+        {
+          signal_id = g_signal_connect (G_OBJECT (widget), "commit",
+                                        G_CALLBACK (terminal_widget_commit), &committed);
         }
-    }
-  else if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
-    {
-      signal_id = g_signal_connect (G_OBJECT (widget), "commit",
-                                    G_CALLBACK (terminal_widget_commit), &committed);
     }
 
   (*GTK_WIDGET_CLASS (terminal_widget_parent_class)->button_press_event) (widget, event);
@@ -433,7 +438,7 @@ terminal_widget_button_press_event (GtkWidget       *widget,
         {
           terminal_widget_context_menu (TERMINAL_WIDGET (widget),
                                         event->button, event->time,
-                                        event->x, event->y);
+                                        (GdkEvent *) event);
         }
     }
 
@@ -452,7 +457,7 @@ terminal_widget_drag_data_received (GtkWidget        *widget,
                                     guint32           drag_time)
 {
   const guint16 *ucs;
-  GdkColor       color;
+  GdkRGBA        color;
   GString       *str;
   GValue         value = { 0, };
   gchar        **uris;
@@ -478,32 +483,32 @@ terminal_widget_drag_data_received (GtkWidget        *widget,
       break;
 
     case TARGET_TEXT_PLAIN:
-      if (selection_data->format != 8 || selection_data->length == 0)
+      if (gtk_selection_data_get_format (selection_data) != 8 || gtk_selection_data_get_length (selection_data) == 0)
         {
           g_printerr (_("Unable to drop selection of type text/plain to terminal: Wrong format (%d) or length (%d)\n"),
-                      selection_data->format, selection_data->length);
+                      gtk_selection_data_get_format (selection_data), gtk_selection_data_get_length (selection_data));
         }
       else
         {
           vte_terminal_feed_child (VTE_TERMINAL (widget),
-                                   (const gchar *) selection_data->data,
-                                   selection_data->length);
+                                   (const gchar *) gtk_selection_data_get_data (selection_data),
+                                   gtk_selection_data_get_length (selection_data));
         }
       break;
 
     case TARGET_MOZ_URL:
-      if (selection_data->format != 8
-          || selection_data->length == 0
-          || (selection_data->length % 2) != 0)
+      if (gtk_selection_data_get_format (selection_data) != 8
+          || gtk_selection_data_get_length (selection_data) == 0
+          || (gtk_selection_data_get_length (selection_data) % 2) != 0)
         {
           g_printerr (_("Unable to drop Mozilla URL on terminal: Wrong format (%d) or length (%d)\n"),
-                      selection_data->format, selection_data->length);
+                      gtk_selection_data_get_format (selection_data), gtk_selection_data_get_length (selection_data));
         }
       else
         {
           str = g_string_new (NULL);
-          ucs = (const guint16 *) selection_data->data;
-          for (n = 0; n < selection_data->length / 2 && ucs[n] != '\n'; ++n)
+          ucs = (const guint16 *) gtk_selection_data_get_data (selection_data);
+          for (n = 0; n < gtk_selection_data_get_length (selection_data) / 2 && ucs[n] != '\n'; ++n)
             g_string_append_unichar (str, (gunichar) ucs[n]);
           filename = g_filename_from_uri (str->str, NULL, NULL);
           if (filename != NULL)
@@ -520,15 +525,15 @@ terminal_widget_drag_data_received (GtkWidget        *widget,
       break;
 
     case TARGET_URI_LIST:
-      if (selection_data->format != 8 || selection_data->length == 0)
+      if (gtk_selection_data_get_format (selection_data) != 8 || gtk_selection_data_get_length (selection_data) == 0)
         {
           g_printerr (_("Unable to drop URI list on terminal: Wrong format (%d) or length (%d)\n"),
-                      selection_data->format, selection_data->length);
+                      gtk_selection_data_get_format (selection_data), gtk_selection_data_get_length (selection_data));
         }
       else
         {
           /* split the text/uri-list */
-          text = g_strndup ((const gchar *) selection_data->data, selection_data->length);
+          text = g_strndup ((const gchar *) gtk_selection_data_get_data (selection_data), gtk_selection_data_get_length (selection_data));
           uris = g_uri_list_extract_uris (text);
           g_free (text);
 
@@ -565,20 +570,22 @@ terminal_widget_drag_data_received (GtkWidget        *widget,
       break;
 
     case TARGET_APPLICATION_X_COLOR:
-      if (selection_data->format != 16 || selection_data->length != 8)
+      if (gtk_selection_data_get_format (selection_data) != 16 || gtk_selection_data_get_length (selection_data) != 8)
         {
           g_printerr (_("Received invalid color data: Wrong format (%d) or length (%d)\n"),
-                      selection_data->format, selection_data->length);
+                      gtk_selection_data_get_format (selection_data), gtk_selection_data_get_length (selection_data));
         }
       else
         {
           /* get the color from the selection data (ignoring the alpha setting) */
-          color.red = ((guint16 *) selection_data->data)[0];
-          color.green = ((guint16 *) selection_data->data)[1];
-          color.blue = ((guint16 *) selection_data->data)[2];
+          const guchar *data = gtk_selection_data_get_data (selection_data);
+          color.red   = (gdouble) data[0] / 65535.;
+          color.green = (gdouble) data[1] / 65535.;
+          color.blue  = (gdouble) data[2] / 65535.;
+          color.alpha = 1.;
 
           /* prepare the value */
-          g_value_init (&value, GDK_TYPE_COLOR);
+          g_value_init (&value, GDK_TYPE_RGBA);
           g_value_set_boxed (&value, &color);
 
           /* change the background to the specified color */
@@ -614,12 +621,10 @@ static gboolean
 terminal_widget_key_press_event (GtkWidget    *widget,
                                  GdkEventKey  *event)
 {
-  GtkAdjustment *adjustment = VTE_TERMINAL (widget)->adjustment;
+  GtkAdjustment *adjustment = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (widget));
   gboolean       scrolling_single_line;
   gboolean       shortcuts_no_menukey;
   gdouble        value;
-  gint           x;
-  gint           y;
 
   /* determine current settings */
   g_object_get (G_OBJECT (TERMINAL_WIDGET (widget)->preferences),
@@ -628,25 +633,24 @@ terminal_widget_key_press_event (GtkWidget    *widget,
                 NULL);
 
   /* popup context menu if "Menu" or "<Shift>F10" is pressed */
-  if (event->keyval == GDK_Menu ||
-      (!shortcuts_no_menukey && (event->state & GDK_SHIFT_MASK) != 0 && event->keyval == GDK_F10))
+  if (event->keyval == GDK_KEY_Menu ||
+      (!shortcuts_no_menukey && (event->state & GDK_SHIFT_MASK) != 0 && event->keyval == GDK_KEY_F10))
     {
-      gtk_widget_get_pointer (widget, &x, &y);
-      terminal_widget_context_menu (TERMINAL_WIDGET (widget), 0, event->time, x, y);
+      terminal_widget_context_menu (TERMINAL_WIDGET (widget), 0, event->time, (GdkEvent *) event);
       return TRUE;
     }
   else if (G_LIKELY (scrolling_single_line))
     {
       /* scroll up one line with "<Shift>Up" */
-      if ((event->state & GDK_SHIFT_MASK) != 0 && (event->keyval == GDK_Up || event->keyval == GDK_KP_Up))
+      if ((event->state & GDK_SHIFT_MASK) != 0 && (event->keyval == GDK_KEY_Up || event->keyval == GDK_KEY_KP_Up))
         {
-          gtk_adjustment_set_value (adjustment, adjustment->value - 1);
+          gtk_adjustment_set_value (adjustment, gtk_adjustment_get_value (adjustment) - 1);
           return TRUE;
         }
       /* scroll down one line with "<Shift>Down" */
-      else if ((event->state & GDK_SHIFT_MASK) != 0 && (event->keyval == GDK_Down || event->keyval == GDK_KP_Down))
+      else if ((event->state & GDK_SHIFT_MASK) != 0 && (event->keyval == GDK_KEY_Down || event->keyval == GDK_KEY_KP_Down))
         {
-          value = MIN (adjustment->value + 1, adjustment->upper - adjustment->page_size);
+          value = MIN (gtk_adjustment_get_value (adjustment) + 1, gtk_adjustment_get_upper (adjustment) - gtk_adjustment_get_page_size (adjustment));
           gtk_adjustment_set_value (adjustment, value);
           return TRUE;
         }
@@ -685,23 +689,26 @@ terminal_widget_open_uri (TerminalWidget *widget,
             break;
 
           case PATTERN_TYPE_EMAIL:
-            if (strncmp (wlink, "mailto:", 7) == 0)
+            if (strncmp (wlink, MAILTO, strlen (MAILTO)) == 0)
               uri = g_strdup (wlink);
             else
-              uri = g_strconcat ("mailto:", wlink, NULL);
+              uri = g_strconcat (MAILTO, wlink, NULL);
             break;
 
           default:
-            goto invalid_tag;
+            g_warning ("Invalid tag specified while trying to open link \"%s\".", wlink);
+            return;
         }
 
       /* try to open the URI with the responsible application */
       screen = gtk_widget_get_screen (GTK_WIDGET (widget));
       if (!gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error))
         {
+          /* escape ampersand symbols, etc. */
+          uri = g_markup_escape_text (uri, -1);
           /* tell the user that we were unable to open the responsible application */
           xfce_dialog_show_error (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (widget))),
-                                  error, _("Failed to open the URL `%s'"), uri);
+                                  error, _("Failed to open the URL '%s'"), uri);
           g_error_free (error);
         }
 
@@ -710,10 +717,6 @@ terminal_widget_open_uri (TerminalWidget *widget,
       /* done */
       return;
     }
-
-invalid_tag:
-
-  g_warning ("Invalid tag specified while trying to open link \"%s\".", wlink);
 }
 
 
