@@ -96,8 +96,7 @@ typedef enum
   ANIMATION_DIR_NONE,
   ANIMATION_DIR_UP,
   ANIMATION_DIR_DOWN,
-}
-TerminalDirection;
+} TerminalDirection;
 
 struct _TerminalWindowDropdown
 {
@@ -221,6 +220,7 @@ terminal_window_dropdown_init (TerminalWindowDropdown *dropdown)
   guint           n;
   const gchar    *name;
   gboolean        keep_open;
+  gboolean        show_borders;
   GtkWidget      *child;
 
   dropdown->rel_width = 0.80;
@@ -240,10 +240,8 @@ terminal_window_dropdown_init (TerminalWindowDropdown *dropdown)
   gtk_container_add (GTK_CONTAINER (dropdown), dropdown->viewport);
   gtk_container_add (GTK_CONTAINER (dropdown->viewport), child);
   g_object_unref (G_OBJECT (child));
-  gtk_widget_show (dropdown->viewport);
 
   /* default window settings */
-  gtk_window_set_resizable (GTK_WINDOW (dropdown), FALSE);
   gtk_window_set_decorated (GTK_WINDOW (dropdown), FALSE);
   gtk_window_set_gravity (GTK_WINDOW (dropdown), GDK_GRAVITY_STATIC);
   gtk_window_set_type_hint (GTK_WINDOW (dropdown), GDK_WINDOW_TYPE_HINT_NORMAL);
@@ -256,32 +254,33 @@ terminal_window_dropdown_init (TerminalWindowDropdown *dropdown)
 
   /* adjust notebook for drop-down usage */
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK (window->notebook), GTK_POS_BOTTOM);
-  gtk_notebook_set_show_border (GTK_NOTEBOOK (window->notebook), TRUE);
+  g_object_get (G_OBJECT (window->preferences), "misc-borders-default", &show_borders, NULL);
+  gtk_notebook_set_show_border (GTK_NOTEBOOK (window->notebook), show_borders);
   terminal_window_notebook_show_tabs (window);
-  terminal_util_set_style_thinkess (window->notebook, 1);
 
   /* actions we don't want */
   action = gtk_action_group_get_action (window->action_group, "show-borders");
   gtk_action_set_visible (action, FALSE);
 
   /* notebook buttons */
-  hbox = gtk_hbox_new (TRUE, 2);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
   gtk_notebook_set_action_widget (GTK_NOTEBOOK (window->notebook), hbox, GTK_PACK_END);
-  gtk_widget_show (hbox);
 
   button = dropdown->keep_open = gtk_toggle_button_new ();
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_widget_set_tooltip_text (button, _("Keep window open when it loses focus"));
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+#if GTK_CHECK_VERSION (3,20,0)
+  gtk_widget_set_focus_on_click (button, FALSE);
+#else
   gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
-  gtk_widget_show (button);
+#endif
 
   g_object_get (G_OBJECT (window->preferences), "dropdown-keep-open-default", &keep_open, NULL);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), keep_open);
 
-  img = gtk_image_new_from_stock (GTK_STOCK_GOTO_BOTTOM, GTK_ICON_SIZE_MENU);
+  img = gtk_image_new_from_icon_name ("go-bottom", GTK_ICON_SIZE_MENU);
   gtk_container_add (GTK_CONTAINER (button), img);
-  gtk_widget_show (img);
 
   action = gtk_action_group_get_action (window->action_group, "preferences");
 
@@ -289,14 +288,16 @@ terminal_window_dropdown_init (TerminalWindowDropdown *dropdown)
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_widget_set_tooltip_text (button, gtk_action_get_tooltip (action));
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+#if GTK_CHECK_VERSION (3,20,0)
+  gtk_widget_set_focus_on_click (button, FALSE);
+#else
   gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
+#endif
   g_signal_connect_swapped (G_OBJECT (button), "clicked",
       G_CALLBACK (gtk_action_activate), action);
-  gtk_widget_show (button);
 
   img = gtk_action_create_icon (action, GTK_ICON_SIZE_MENU);
   gtk_container_add (GTK_CONTAINER (button), img);
-  gtk_widget_show (img);
 
   /* connect bindings */
   for (n = 1; n < N_PROPERTIES; n++)
@@ -336,13 +337,13 @@ terminal_window_dropdown_set_property (GObject      *object,
       break;
 
     case PROP_DROPDOWN_OPACITY:
-      screen = gtk_window_get_screen (GTK_WINDOW (dropdown));
+      screen = gtk_widget_get_screen (GTK_WIDGET (dropdown));
       if (gdk_screen_is_composited (screen))
         opacity = g_value_get_uint (value) / 100.0;
       else
         opacity = 1.00;
 
-      gtk_window_set_opacity (GTK_WINDOW (dropdown), opacity);
+      gtk_widget_set_opacity (GTK_WIDGET (dropdown), opacity);
       return;
 
     case PROP_DROPDOWN_STATUS_ICON:
@@ -351,7 +352,6 @@ terminal_window_dropdown_set_property (GObject      *object,
           if (dropdown->status_icon == NULL)
             {
               dropdown->status_icon = gtk_status_icon_new_from_icon_name ("utilities-terminal");
-              gtk_status_icon_set_name (dropdown->status_icon, PACKAGE_NAME);
               gtk_status_icon_set_title (dropdown->status_icon, _("Drop-down Terminal"));
               gtk_status_icon_set_tooltip_text (dropdown->status_icon, _("Toggle Drop-down Terminal"));
               g_signal_connect (G_OBJECT (dropdown->status_icon), "button-press-event",
@@ -531,7 +531,6 @@ terminal_window_dropdown_status_icon_popup_menu (GtkStatusIcon          *status_
 {
   GtkActionGroup *group = TERMINAL_WINDOW (dropdown)->action_group;
   GtkWidget      *menu;
-  GtkWidget      *mi;
   GtkAction      *action;
 
   menu = gtk_menu_new ();
@@ -539,19 +538,17 @@ terminal_window_dropdown_status_icon_popup_menu (GtkStatusIcon          *status_
       G_CALLBACK (gtk_widget_destroy), NULL);
 
   action = gtk_action_group_get_action (group, "preferences");
-  mi = gtk_action_create_menu_item (action);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-  gtk_widget_show (mi);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         gtk_action_create_menu_item (action));
 
-  mi = gtk_separator_menu_item_new ();
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-  gtk_widget_show (mi);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         gtk_separator_menu_item_new ());
 
   action = gtk_action_group_get_action (group, "close-window");
-  mi = gtk_action_create_menu_item (action);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-  gtk_widget_show (mi);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         gtk_action_create_menu_item (action));
 
+  gtk_widget_show_all (menu);
   gtk_menu_popup (GTK_MENU (menu),
                   NULL, NULL,
                   NULL, NULL,
@@ -570,13 +567,13 @@ terminal_window_dropdown_animate_down (gpointer data)
   gint                    viewport_h;
 
   /* decrease each interval */
-  gtk_widget_size_request (window->vbox, &req1);
+  gtk_widget_get_preferred_size (window->vbox, &req1, NULL);
   step_size = req1.height * ANIMATION_FPS / dropdown->animation_time;
   if (step_size < 1)
     step_size = 1;
 
   /* new viewport size */
-  gtk_widget_size_request (dropdown->viewport, &req2);
+  gtk_widget_get_preferred_size (dropdown->viewport, &req2, NULL);
   viewport_h = req2.height + step_size;
   if (viewport_h > req1.height)
     viewport_h = req1.height;
@@ -605,8 +602,7 @@ terminal_window_dropdown_animate_up (gpointer data)
   TerminalWindowDropdown *dropdown = TERMINAL_WINDOW_DROPDOWN (data);
   TerminalWindow         *window = TERMINAL_WINDOW (data);
   GtkRequisition          req1, req2;
-  gint                    step_size;
-  gint                    viewport_h;
+  gint                    step_size, viewport_h, min_size;
   GdkRectangle            rect;
 
   /* get window size */
@@ -619,7 +615,7 @@ terminal_window_dropdown_animate_up (gpointer data)
     }
   else
     {
-      gtk_widget_size_request (TERMINAL_WINDOW (dropdown)->vbox, &req1);
+      gtk_widget_get_preferred_size (window->vbox, &req1, NULL);
     }
 
   /* decrease each interval */
@@ -628,22 +624,37 @@ terminal_window_dropdown_animate_up (gpointer data)
     step_size = 1;
 
   /* new viewport size */
-  gtk_widget_size_request (dropdown->viewport, &req2);
+  gtk_widget_get_preferred_size (dropdown->viewport, &req2, NULL);
   viewport_h = req2.height - step_size;
 
-  if (viewport_h < step_size)
+  /* sizes of the widgets that cannot be shrunk */
+  gtk_widget_get_preferred_size (window->notebook, &req2, NULL);
+  min_size = req2.height;
+  if (window->menubar != NULL
+      && gtk_widget_get_visible (window->menubar))
+    {
+      gtk_widget_get_preferred_size (window->menubar, &req2, NULL);
+      min_size += req2.height;
+    }
+  if (window->toolbar != NULL
+      && gtk_widget_get_visible (window->toolbar))
+    {
+      gtk_widget_get_preferred_size (window->toolbar, &req2, NULL);
+      min_size += req2.height;
+    }
+
+  if (viewport_h < min_size)
     {
       /* animation complete */
       gtk_widget_hide (GTK_WIDGET (dropdown));
       return FALSE;
     }
-  else
-    {
-      /* resize viewport */
-      gtk_widget_set_size_request (dropdown->viewport, req1.width, viewport_h);
-      gtk_window_resize (GTK_WINDOW (dropdown), req1.width, viewport_h);
-      return TRUE;
-    }
+
+  /* resize window */
+  gtk_widget_set_size_request (dropdown->viewport, req1.width, viewport_h);
+  gtk_widget_set_size_request (window->vbox, req1.width, viewport_h);
+  gtk_window_resize (GTK_WINDOW (dropdown), req1.width, viewport_h);
+  return TRUE;
 }
 
 
@@ -687,9 +698,8 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
   gint               w, h;
   GdkRectangle       monitor_geo;
   gint               x_dest, y_dest;
-  gint               xpad, ypad;
   glong              char_width, char_height;
-  GtkRequisition     req1, req2;
+  GtkRequisition     req1;
   gboolean           move_to_active;
   gboolean           visible;
   gint               viewport_h;
@@ -720,7 +730,7 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
   gtk_window_set_screen (GTK_WINDOW (dropdown), dropdown->screen);
 
   /* get terminal size */
-  terminal_screen_get_geometry (window->active, &char_width, &char_height, &xpad, &ypad);
+  terminal_screen_get_geometry (window->active, &char_width, &char_height, NULL, NULL);
 
   /* correct padding with notebook size */
   if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (window->action_fullscreen)))
@@ -738,29 +748,6 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
       /* calculate size */
       w = monitor_geo.width * dropdown->rel_width;
       h = monitor_geo.height * dropdown->rel_height;
-
-      gtk_widget_size_request (window->notebook, &req1);
-      gtk_widget_size_request (GTK_WIDGET (window->active), &req2);
-      xpad += MAX (req1.width - req2.width, 0);
-      ypad += MAX (req1.height - req2.height, 0);
-
-      if (window->menubar != NULL
-          && gtk_widget_get_visible (window->menubar))
-        {
-          gtk_widget_size_request (window->menubar, &req2);
-          ypad += req2.height;
-        }
-
-      if (window->toolbar != NULL
-          && gtk_widget_get_visible (window->toolbar))
-        {
-          gtk_widget_size_request (window->toolbar, &req2);
-          ypad += req2.height;
-        }
-
-      /* minimize to fit terminal charaters */
-      w -= (w - xpad) % char_width;
-      h -= (h - ypad) % char_height;
     }
 
   /* viewport size if not animated */
@@ -777,7 +764,7 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
       else if (old_animation_dir == ANIMATION_DIR_UP)
         {
           /* pick up where we aborted */
-          gtk_widget_size_request (dropdown->viewport, &req1);
+          gtk_widget_get_preferred_size (dropdown->viewport, &req1, NULL);
           viewport_h = req1.height;
         }
     }
@@ -795,7 +782,10 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
 
   /* show window */
   if (!visible)
-    gtk_window_present_with_time (GTK_WINDOW (dropdown), timestamp);
+    {
+      gtk_widget_show_all (GTK_WIDGET (dropdown));
+      gtk_window_present_with_time (GTK_WINDOW (dropdown), timestamp);
+    }
 
   /* force focus to the window */
   terminal_util_activate_window (GTK_WINDOW (dropdown));
@@ -986,7 +976,7 @@ terminal_window_dropdown_get_size (TerminalWindowDropdown *dropdown,
   terminal_screen_get_geometry (screen, &char_width, &char_height, &xpad, &ypad);
 
   /* correct padding with visible widgets */
-  gtk_widget_size_request (TERMINAL_WINDOW (dropdown)->vbox, &req);
+  gtk_widget_get_preferred_size (TERMINAL_WINDOW (dropdown)->vbox, &req, NULL);
   xpad += 2;
   ypad += req.height;
 
