@@ -34,7 +34,6 @@
 
 #include <libxfce4ui/libxfce4ui.h>
 
-#include <gdk/gdkkeysyms.h>
 #if defined(GDK_WINDOWING_X11)
 #include <gdk/gdkx.h>
 #endif
@@ -86,11 +85,11 @@ static gboolean        terminal_window_accel_activate                (GtkAccelGr
 static void            terminal_window_update_actions                (TerminalWindow         *window);
 static void            terminal_window_rebuild_tabs_menu             (TerminalWindow         *window);
 static void            terminal_window_notebook_page_switched        (GtkNotebook            *notebook,
-                                                                      GtkNotebookPage        *page,
+                                                                      GtkWidget              *page,
                                                                       guint                   page_num,
                                                                       TerminalWindow         *window);
 static void            terminal_window_notebook_page_reordered       (GtkNotebook            *notebook,
-                                                                      GtkNotebookPage        *page,
+                                                                      GtkWidget              *child,
                                                                       guint                   page_num,
                                                                       TerminalWindow         *window);
 static void            terminal_window_notebook_page_added           (GtkNotebook            *notebook,
@@ -156,6 +155,12 @@ static void            terminal_window_action_show_borders           (GtkToggleA
                                                                       TerminalWindow         *window);
 static void            terminal_window_action_fullscreen             (GtkToggleAction        *action,
                                                                       TerminalWindow         *window);
+static void            terminal_window_action_zoom_in                (GtkAction              *action,
+                                                                      TerminalWindow         *window);
+static void            terminal_window_action_zoom_out               (GtkAction              *action,
+                                                                      TerminalWindow         *window);
+static void            terminal_window_action_zoom_reset             (GtkAction              *action,
+                                                                      TerminalWindow         *window);
 static void            terminal_window_action_prev_tab               (GtkAction              *action,
                                                                       TerminalWindow         *window);
 static void            terminal_window_action_next_tab               (GtkAction              *action,
@@ -182,12 +187,13 @@ static void            terminal_window_action_contents               (GtkAction
                                                                       TerminalWindow         *window);
 static void            terminal_window_action_about                  (GtkAction              *action,
                                                                       TerminalWindow         *window);
+static void            terminal_window_zoom_update_screens           (TerminalWindow         *window);
 
 
 
-static guint         window_signals[LAST_SIGNAL];
-static gconstpointer window_notebook_group = PACKAGE_NAME;
-static GQuark        tabs_menu_action_quark = 0;
+static guint   window_signals[LAST_SIGNAL];
+static gchar   *window_notebook_group = PACKAGE_NAME;
+static GQuark  tabs_menu_action_quark = 0;
 
 
 
@@ -197,31 +203,33 @@ static const GtkActionEntry action_entries[] =
     { "new-tab", "tab-new", N_ ("Open _Tab"), "<control><shift>t", N_ ("Open a new terminal tab"), G_CALLBACK (terminal_window_action_new_tab), },
     { "new-window", "window-new", N_ ("Open T_erminal"), "<control><shift>n", N_ ("Open a new terminal window"), G_CALLBACK (terminal_window_action_new_window), },
     { "detach-tab", NULL, N_ ("_Detach Tab"), "<control><shift>d", NULL, G_CALLBACK (terminal_window_action_detach_tab), },
-    { "close-tab", GTK_STOCK_CLOSE, N_ ("Close T_ab"), "<control><shift>w", NULL, G_CALLBACK (terminal_window_action_close_tab), },
-    { "close-window", GTK_STOCK_QUIT, N_ ("Close _Window"), "<control><shift>q", NULL, G_CALLBACK (terminal_window_action_close_window), },
+    { "close-tab", "window-close", N_ ("Close T_ab"), "<control><shift>w", NULL, G_CALLBACK (terminal_window_action_close_tab), },
+    { "close-window", "application-exit", N_ ("Close _Window"), "<control><shift>q", NULL, G_CALLBACK (terminal_window_action_close_window), },
   { "edit-menu", NULL, N_ ("_Edit"), NULL, NULL, NULL, },
-    { "copy", GTK_STOCK_COPY, N_ ("_Copy"), "<control><shift>c", N_ ("Copy to clipboard"), G_CALLBACK (terminal_window_action_copy), },
-    { "paste", GTK_STOCK_PASTE, N_ ("_Paste"), "<control><shift>v", N_ ("Paste from clipboard"), G_CALLBACK (terminal_window_action_paste), },
+    { "copy", "edit-copy", N_ ("_Copy"), "<control><shift>c", N_ ("Copy to clipboard"), G_CALLBACK (terminal_window_action_copy), },
+    { "paste", "edit-paste", N_ ("_Paste"), "<control><shift>v", N_ ("Paste from clipboard"), G_CALLBACK (terminal_window_action_paste), },
     { "paste-selection", NULL, N_ ("Paste _Selection"), NULL, NULL, G_CALLBACK (terminal_window_action_paste_selection), },
-    { "select-all", GTK_STOCK_SELECT_ALL, N_ ("Select _All"), "<control><shift>a", NULL, G_CALLBACK (terminal_window_action_select_all), },
-    { "preferences", GTK_STOCK_PREFERENCES, N_ ("Pr_eferences..."), NULL, N_ ("Open the preferences dialog"), G_CALLBACK (terminal_window_action_prefs), },
+    { "select-all", "edit-select-all", N_ ("Select _All"), "<control><shift>a", NULL, G_CALLBACK (terminal_window_action_select_all), },
+    { "preferences", "preferences-system", N_ ("Pr_eferences..."), NULL, N_ ("Open the preferences dialog"), G_CALLBACK (terminal_window_action_prefs), },
   { "view-menu", NULL, N_ ("_View"), NULL, NULL, NULL, },
+    { "zoom-in", "zoom-in", N_ ("Zoom _In"), "<control>plus", N_ ("Zoom in with larger font"), G_CALLBACK (terminal_window_action_zoom_in), },
+    { "zoom-out", "zoom-out", N_ ("Zoom _Out"), "<control>minus", N_ ("Zoom out with smaller font"), G_CALLBACK (terminal_window_action_zoom_out), },
+    { "zoom-reset", "zoom-original", N_ ("_Normal Size"), "<control>0", N_ ("Zoom to default size"), G_CALLBACK (terminal_window_action_zoom_reset), },
   { "terminal-menu", NULL, N_ ("_Terminal"), NULL, NULL, NULL, },
-    { "set-title", NULL, N_ ("_Set Title..."), NULL, NULL, G_CALLBACK (terminal_window_action_set_title), },
-    { "search", GTK_STOCK_FIND, N_ ("_Find..."), "<control><shift>f", NULL, G_CALLBACK (terminal_window_action_search), },
+    { "set-title", NULL, N_ ("_Set Title..."), "<control><shift>s", NULL, G_CALLBACK (terminal_window_action_set_title), },
+    { "search", "edit-find", N_ ("_Find..."), "<control><shift>f", N_ ("Search terminal contents"), G_CALLBACK (terminal_window_action_search), },
     { "search-next", NULL, N_ ("Find Ne_xt"), NULL, NULL, G_CALLBACK (terminal_window_action_search_next), },
     { "search-prev", NULL, N_ ("Find Pre_vious"), NULL, NULL, G_CALLBACK (terminal_window_action_search_prev), },
     { "reset", NULL, N_ ("_Reset"), NULL, NULL, G_CALLBACK (terminal_window_action_reset), },
     { "reset-and-clear", NULL, N_ ("_Clear Scrollback and Reset"), NULL, NULL, G_CALLBACK (terminal_window_action_reset_and_clear), },
   { "tabs-menu", NULL, N_ ("T_abs"), NULL, NULL, NULL, },
-    { "prev-tab", GTK_STOCK_GO_BACK, N_ ("_Previous Tab"), "<Control>Page_Up", N_ ("Switch to previous tab"), G_CALLBACK (terminal_window_action_prev_tab), },
-    { "next-tab", GTK_STOCK_GO_FORWARD, N_ ("_Next Tab"), "<Control>Page_Down", N_ ("Switch to next tab"), G_CALLBACK (terminal_window_action_next_tab), },
-    { "move-tab-left", NULL, N_ ("Move Tab _Left"), NULL, NULL, G_CALLBACK (terminal_window_action_move_tab_left), },
-    { "move-tab-right", NULL, N_ ("Move Tab _Right"), NULL, NULL, G_CALLBACK (terminal_window_action_move_tab_right), },
+    { "prev-tab", "go-previous", N_ ("_Previous Tab"), "<control>Page_Up", N_ ("Switch to previous tab"), G_CALLBACK (terminal_window_action_prev_tab), },
+    { "next-tab", "go-next", N_ ("_Next Tab"), "<control>Page_Down", N_ ("Switch to next tab"), G_CALLBACK (terminal_window_action_next_tab), },
+    { "move-tab-left", NULL, N_ ("Move Tab _Left"), "<control><shift>Page_Up", NULL, G_CALLBACK (terminal_window_action_move_tab_left), },
+    { "move-tab-right", NULL, N_ ("Move Tab _Right"), "<control><shift>Page_Down", NULL, G_CALLBACK (terminal_window_action_move_tab_right), },
   { "help-menu", NULL, N_ ("_Help"), NULL, NULL, NULL, },
-    { "contents", GTK_STOCK_HELP, N_ ("_Contents"), "F1", N_ ("Display help contents"), G_CALLBACK (terminal_window_action_contents), },
-    { "about", GTK_STOCK_ABOUT, N_ ("_About"), NULL, NULL, G_CALLBACK (terminal_window_action_about), },
-  { "input-methods", NULL, N_ ("_Input Methods"), NULL, NULL, NULL, },
+    { "contents", "help-browser", N_ ("_Contents"), "F1", N_ ("Display help contents"), G_CALLBACK (terminal_window_action_contents), },
+    { "about", "help-about", N_ ("_About"), NULL, NULL, G_CALLBACK (terminal_window_action_about), },
 };
 
 static const GtkToggleActionEntry toggle_action_entries[] =
@@ -229,7 +237,7 @@ static const GtkToggleActionEntry toggle_action_entries[] =
   { "show-menubar", NULL, N_ ("Show _Menubar"), NULL, N_ ("Show/hide the menubar"), G_CALLBACK (terminal_window_action_show_menubar), FALSE, },
   { "show-toolbar", NULL, N_ ("Show _Toolbar"), NULL, N_ ("Show/hide the toolbar"), G_CALLBACK (terminal_window_action_show_toolbar), FALSE, },
   { "show-borders", NULL, N_ ("Show Window _Borders"), NULL, N_ ("Show/hide the window decorations"), G_CALLBACK (terminal_window_action_show_borders), TRUE, },
-  { "fullscreen", GTK_STOCK_FULLSCREEN, N_ ("_Fullscreen"), "F11", N_ ("Toggle fullscreen mode"), G_CALLBACK (terminal_window_action_fullscreen), FALSE, },
+  { "fullscreen", "view-fullscreen", N_ ("_Fullscreen"), "F11", N_ ("Toggle fullscreen mode"), G_CALLBACK (terminal_window_action_fullscreen), FALSE, },
 };
 
 
@@ -290,15 +298,18 @@ terminal_window_init (TerminalWindow *window)
   GtkWidget      *vbox;
   gboolean        always_show_tabs;
   GdkScreen      *screen;
-  GdkColormap    *colormap;
+  GdkVisual      *visual;
 
   window->preferences = terminal_preferences_get ();
 
+  window->font = NULL;
+  window->zoom = TERMINAL_ZOOM_LEVEL_DEFAULT;
+
   /* try to set the rgba colormap so vte can use real transparency */
   screen = gtk_window_get_screen (GTK_WINDOW (window));
-  colormap = gdk_screen_get_rgba_colormap (screen);
-  if (colormap != NULL)
-    gtk_widget_set_colormap (GTK_WIDGET (window), colormap);
+  visual = gdk_screen_get_rgba_visual (screen);
+  if (visual != NULL)
+    gtk_widget_set_visual (GTK_WIDGET (window), visual);
 
   window->action_group = gtk_action_group_new ("terminal-window");
   gtk_action_group_set_translation_domain (window->action_group,
@@ -321,27 +332,19 @@ terminal_window_init (TerminalWindow *window)
   g_signal_connect_after (G_OBJECT (accel_group), "accel-activate",
       G_CALLBACK (terminal_window_accel_activate), window);
 
-  window->vbox = vbox = gtk_vbox_new (FALSE, 0);
+  window->vbox = vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
   gtk_container_add (GTK_CONTAINER (window), vbox);
-  gtk_widget_show (vbox);
 
   /* allocate the notebook for the terminal screens */
   g_object_get (G_OBJECT (window->preferences), "misc-always-show-tabs", &always_show_tabs, NULL);
   window->notebook = g_object_new (GTK_TYPE_NOTEBOOK,
-                                   "homogeneous", TRUE,
                                    "scrollable", TRUE,
                                    "show-border", FALSE,
                                    "show-tabs", always_show_tabs,
-                                   "tab-hborder", 0,
-                                   "tab-vborder", 0,
                                    NULL);
 
-  /* hide the ugly terminal border when tabs are shown */
-  terminal_util_set_style_thinkess (window->notebook, 0);
-
   /* set the notebook group id */
-  gtk_notebook_set_group (GTK_NOTEBOOK (window->notebook),
-                          (gpointer) window_notebook_group);
+  gtk_notebook_set_group_name (GTK_NOTEBOOK (window->notebook), window_notebook_group);
 
   /* signals */
   g_signal_connect (G_OBJECT (window->notebook), "switch-page",
@@ -360,7 +363,7 @@ terminal_window_init (TerminalWindow *window)
       G_CALLBACK (terminal_window_notebook_button_release_event), window);
 
   gtk_box_pack_start (GTK_BOX (vbox), window->notebook, TRUE, TRUE, 0);
-  gtk_widget_show (window->notebook);
+  gtk_widget_show_all (vbox);
 
   /* create encoding action */
   window->encoding_action = terminal_encoding_action_new ("set-encoding", _("Set _Encoding"));
@@ -369,7 +372,7 @@ terminal_window_init (TerminalWindow *window)
       G_CALLBACK (terminal_window_action_set_encoding), window);
 
   /* cache action pointers */
-  window->action_detah_tab = gtk_action_group_get_action (window->action_group, "detach-tab");
+  window->action_detach_tab = gtk_action_group_get_action (window->action_group, "detach-tab");
   window->action_close_tab = gtk_action_group_get_action (window->action_group, "close-tab");
   window->action_prev_tab = gtk_action_group_get_action (window->action_group, "prev-tab");
   window->action_next_tab = gtk_action_group_get_action (window->action_group, "next-tab");
@@ -382,7 +385,7 @@ terminal_window_init (TerminalWindow *window)
 
 #if defined(GDK_WINDOWING_X11)
   /* setup fullscreen mode */
-  if (!gdk_net_wm_supports (gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN", FALSE)))
+  if (!gdk_x11_screen_supports_net_wm_hint (screen, gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN", FALSE)))
     gtk_action_set_sensitive (window->action_fullscreen, FALSE);
 #endif
 }
@@ -400,6 +403,7 @@ terminal_window_finalize (GObject *object)
   g_object_unref (G_OBJECT (window->encoding_action));
 
   g_slist_free (window->tabs_menu_actions);
+  g_free (window->font);
 
   (*G_OBJECT_CLASS (terminal_window_parent_class)->finalize) (object);
 }
@@ -501,34 +505,29 @@ terminal_window_confirm_close (TerminalWindow *window)
 
   dialog = gtk_dialog_new_with_buttons (_("Warning"), GTK_WINDOW (window),
                                         GTK_DIALOG_DESTROY_WITH_PARENT
-                                        | GTK_DIALOG_NO_SEPARATOR
                                         | GTK_DIALOG_MODAL,
+                                        _("_Cancel"),
+                                        GTK_RESPONSE_CANCEL,
                                         NULL);
 
-  gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-
-  button = xfce_gtk_button_new_mixed (GTK_STOCK_CLOSE, _("Close T_ab"));
+  button = xfce_gtk_button_new_mixed ("window-close", _("Close T_ab"));
   gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_CLOSE);
-  gtk_widget_show (button);
 
-  button = xfce_gtk_button_new_mixed (GTK_STOCK_QUIT, _("Close _Window"));
+  button = xfce_gtk_button_new_mixed ("application-exit", _("Close _Window"));
   gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_YES);
   gtk_widget_grab_focus (button);
-  gtk_widget_show (button);
 
-  hbox = gtk_hbox_new (FALSE, 6);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 8);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, TRUE, TRUE, 0);
-  gtk_widget_show (hbox);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), hbox, TRUE, TRUE, 0);
 
-  image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_DIALOG);
-  gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
+  image = gtk_image_new_from_icon_name ("dialog-warning", GTK_ICON_SIZE_DIALOG);
+  gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
+  gtk_widget_set_valign (image, GTK_ALIGN_START);
   gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-  gtk_widget_show (image);
 
-  vbox = gtk_vbox_new (FALSE, 6);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
-  gtk_widget_show (vbox);
 
   message = g_strdup_printf (_("This window has %d tabs open. Closing this window\n"
                                "will also close all its tabs."), n_tabs);
@@ -542,13 +541,13 @@ terminal_window_confirm_close (TerminalWindow *window)
                         "xalign", 0.0,
                         NULL);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 0);
-  gtk_widget_show (label);
 
   g_free (markup);
 
   checkbox = gtk_check_button_new_with_mnemonic (_("Do _not ask me again"));
   gtk_box_pack_start (GTK_BOX (vbox), checkbox, FALSE, FALSE, 0);
-  gtk_widget_show (checkbox);
+
+  gtk_widget_show_all (dialog);
 
   response = gtk_dialog_run (GTK_DIALOG (dialog));
   if (response == GTK_RESPONSE_YES)
@@ -668,11 +667,11 @@ terminal_window_update_actions (TerminalWindow *window)
   /* determine the number of pages */
   n_pages = gtk_notebook_get_n_pages (notebook);
 
-  /* "Detach Tab" is only sensitive if we have atleast two pages */
-  gtk_action_set_sensitive (window->action_detah_tab, (n_pages > 1));
+  /* "Detach Tab" is only sensitive if we have at least two pages */
+  gtk_action_set_sensitive (window->action_detach_tab, (n_pages > 1));
 
-  /* ... same for the "Close Tab" action */
-  gtk_action_set_sensitive (window->action_close_tab, (n_pages > 1));
+  /* always allow "Close Tab" action (bug 12490) */
+  gtk_action_set_sensitive (window->action_close_tab, TRUE);
 
   /* update the actions for the current terminal screen */
  if (G_LIKELY (window->active != NULL))
@@ -784,7 +783,7 @@ terminal_window_rebuild_tabs_menu (TerminalWindow *window)
 
 static void
 terminal_window_notebook_page_switched (GtkNotebook     *notebook,
-                                        GtkNotebookPage *page,
+                                        GtkWidget       *page,
                                         guint            page_num,
                                         TerminalWindow  *window)
 {
@@ -793,7 +792,9 @@ terminal_window_notebook_page_switched (GtkNotebook     *notebook,
   const gchar    *encoding;
 
   /* get the new active page */
-  active = TERMINAL_SCREEN (gtk_notebook_get_nth_page (notebook, page_num));
+  active = TERMINAL_SCREEN (page);
+
+  terminal_return_if_fail (window == NULL);
   terminal_return_if_fail (active == NULL || TERMINAL_IS_SCREEN (active));
 
   /* only update when really changed */
@@ -829,7 +830,7 @@ terminal_window_notebook_page_switched (GtkNotebook     *notebook,
 
 static void
 terminal_window_notebook_page_reordered (GtkNotebook     *notebook,
-                                         GtkNotebookPage *page,
+                                         GtkWidget       *child,
                                          guint            page_num,
                                          TerminalWindow  *window)
 {
@@ -901,7 +902,9 @@ terminal_window_notebook_page_removed (GtkNotebook    *notebook,
                                        guint           page_num,
                                        TerminalWindow *window)
 {
-  gint npages;
+  GtkWidget *new_page;
+  gint       new_page_num;
+  gint       npages;
 
   terminal_return_if_fail (TERMINAL_IS_SCREEN (child));
   terminal_return_if_fail (TERMINAL_IS_WINDOW (window));
@@ -925,26 +928,39 @@ terminal_window_notebook_page_removed (GtkNotebook    *notebook,
     {
       /* no tabs, destroy the window */
       gtk_widget_destroy (GTK_WIDGET (window));
+      return;
     }
-  else
-    {
-      /* show the tabs when needed */
-      terminal_window_notebook_show_tabs (window);
 
-      /* regenerate the "Go" menu */
-      terminal_window_rebuild_tabs_menu (window);
-    }
+  /* show the tabs when needed */
+  terminal_window_notebook_show_tabs (window);
+
+  /* regenerate the "Go" menu */
+  terminal_window_rebuild_tabs_menu (window);
+
+  /* send a signal about switching to another tab */
+  new_page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (window->notebook));
+  new_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (window->notebook), new_page_num);
+  terminal_window_notebook_page_switched (notebook, new_page, new_page_num, window);
 }
 
+static gboolean
+terminal_window_notebook_event_in_allocation (gint event_x,
+                                              gint event_y,
+                                              GtkWidget *widget)
+{
+  cairo_rectangle_int_t allocation;
+  gtk_widget_get_allocation (widget, &allocation);
 
+  if (event_x >= allocation.x \
+      && event_x <= allocation.x + allocation.width \
+      && event_y >= allocation.y \
+      && event_y <= allocation.y + allocation.height)
+    {
+      return TRUE;
+    }
 
-#define EVENT_IN_ALLOCATION(event_x,event_y,widget) \
-  ((event_x) >= (widget)->allocation.x  \
-  && (event_x) <= (widget)->allocation.x + (widget)->allocation.width \
-  && (event_y) >= (widget)->allocation.y \
-  && (event_y) <= (widget)->allocation.y + (widget)->allocation.height)
-
-
+  return FALSE;
+}
 
 static gboolean
 terminal_window_notebook_button_press_event (GtkNotebook    *notebook,
@@ -969,7 +985,7 @@ terminal_window_notebook_button_press_event (GtkNotebook    *notebook,
         {
           /* check if the user double-clicked on the label */
           label = gtk_notebook_get_tab_label (notebook, GTK_WIDGET (window->active));
-          if (EVENT_IN_ALLOCATION (x, y, label))
+          if (terminal_window_notebook_event_in_allocation (x, y, label))
             {
               terminal_window_action_set_title (NULL, window);
               return TRUE;
@@ -982,7 +998,7 @@ terminal_window_notebook_button_press_event (GtkNotebook    *notebook,
       while ((page = gtk_notebook_get_nth_page (notebook, page_num)) != NULL)
         {
           label = gtk_notebook_get_tab_label (notebook, page);
-          if (EVENT_IN_ALLOCATION (x, y, label))
+          if (terminal_window_notebook_event_in_allocation (x, y, label))
             break;
           page_num++;
         }
@@ -1040,7 +1056,7 @@ terminal_window_notebook_drag_data_received (GtkWidget        *widget,
                                              gint              y,
                                              GtkSelectionData *selection_data,
                                              guint             info,
-                                             guint32           drag_time,
+                                             guint             drag_time,
                                              TerminalWindow   *window)
 {
   GtkWidget  *notebook;
@@ -1048,6 +1064,7 @@ terminal_window_notebook_drag_data_received (GtkWidget        *widget,
   GtkWidget  *child, *label;
   gint        i, n_pages;
   gboolean    succeed = FALSE;
+  cairo_rectangle_int_t allocation;
 
   terminal_return_if_fail (TERMINAL_IS_WINDOW (window));
   terminal_return_if_fail (TERMINAL_IS_SCREEN (widget));
@@ -1060,7 +1077,7 @@ terminal_window_notebook_drag_data_received (GtkWidget        *widget,
       terminal_return_if_fail (GTK_IS_NOTEBOOK (notebook));
 
       /* get the dragged screen */
-      screen = (GtkWidget **) selection_data->data;
+      screen = (GtkWidget **) gtk_selection_data_get_data (selection_data);
       if (!TERMINAL_IS_SCREEN (*screen))
         goto leave;
 
@@ -1077,9 +1094,10 @@ terminal_window_notebook_drag_data_received (GtkWidget        *widget,
           /* get the child label */
           child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (window->notebook), i);
           label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (window->notebook), child);
+          gtk_widget_get_allocation (label, &allocation);
 
           /* break if we have a matching drop position */
-          if (x < (label->allocation.x + label->allocation.width / 2))
+          if (x < (allocation.x + allocation.width / 2))
             break;
         }
 
@@ -1096,7 +1114,11 @@ terminal_window_notebook_drag_data_received (GtkWidget        *widget,
           g_object_ref (G_OBJECT (window));
 
           /* remove the document from the source notebook */
+#if GTK_CHECK_VERSION (3,16,0)
+          gtk_notebook_detach_tab (GTK_NOTEBOOK (notebook), *screen);
+#else
           gtk_container_remove (GTK_CONTAINER (notebook), *screen);
+#endif
 
           /* add the screen to the new window */
           terminal_window_add (window, TERMINAL_SCREEN (*screen));
@@ -1143,7 +1165,11 @@ terminal_window_notebook_create_window (GtkNotebook    *notebook,
       g_object_ref (G_OBJECT (screen));
 
       /* remove screen from active window */
-      gtk_container_remove (GTK_CONTAINER (window->notebook), child);
+      #if GTK_CHECK_VERSION (3,16,0)
+        gtk_notebook_detach_tab (notebook, child);
+      #else
+        gtk_container_remove (GTK_CONTAINER (notebook), child);
+      #endif
 
       /* create new window with the screen */
       g_signal_emit (G_OBJECT (window), window_signals[NEW_WINDOW_WITH_SCREEN], 0, screen, x, y);
@@ -1162,27 +1188,11 @@ terminal_window_get_context_menu (TerminalScreen  *screen,
                                   TerminalWindow  *window)
 {
   GtkWidget *popup;
-  GtkWidget *menu;
-  GtkWidget *item;
 
   if (G_UNLIKELY (screen != window->active))
     return NULL;
 
   popup = gtk_ui_manager_get_widget (window->ui_manager, "/popup-menu");
-  if (G_LIKELY (popup != NULL))
-    {
-      item = gtk_ui_manager_get_widget (window->ui_manager, "/popup-menu/input-methods");
-      if (G_LIKELY (item != NULL && GTK_IS_MENU_ITEM (item)))
-        {
-          /* append input methods */
-          menu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (item));
-          if (G_LIKELY (menu != NULL))
-            gtk_widget_destroy (menu);
-          menu = gtk_menu_new ();
-          terminal_screen_im_append_menuitems (screen, GTK_MENU_SHELL (menu));
-          gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
-        }
-    }
 
   return popup;
 }
@@ -1369,6 +1379,8 @@ terminal_window_action_prefs (GtkAction      *action,
   if (window->preferences_dialog != NULL)
     {
       gtk_window_set_transient_for (GTK_WINDOW (window->preferences_dialog), GTK_WINDOW (window));
+      if (window->drop_down)
+        gtk_window_set_keep_above (GTK_WINDOW (window->preferences_dialog), TRUE);
       gtk_window_present (GTK_WINDOW (window->preferences_dialog));
     }
 }
@@ -1456,6 +1468,51 @@ terminal_window_action_fullscreen (GtkToggleAction *action,
 
 
 
+static void
+terminal_window_action_zoom_in (GtkAction     *action,
+                               TerminalWindow *window)
+{
+  terminal_return_if_fail (window->active != NULL);
+
+  if (window->zoom < TERMINAL_ZOOM_LEVEL_MAXIMUM)
+    {
+      ++window->zoom;
+      terminal_window_zoom_update_screens (window);
+    }
+}
+
+
+
+static void
+terminal_window_action_zoom_out (GtkAction      *action,
+                                 TerminalWindow *window)
+{
+  terminal_return_if_fail (window->active != NULL);
+
+  if (window->zoom > TERMINAL_ZOOM_LEVEL_MINIMUM)
+    {
+      --window->zoom;
+      terminal_window_zoom_update_screens (window);
+    }
+}
+
+
+
+static void
+terminal_window_action_zoom_reset (GtkAction      *action,
+                                   TerminalWindow *window)
+{
+  terminal_return_if_fail (window->active != NULL);
+
+  if (window->zoom != TERMINAL_ZOOM_LEVEL_DEFAULT)
+    {
+      window->zoom = TERMINAL_ZOOM_LEVEL_DEFAULT;
+      terminal_window_zoom_update_screens (window);
+    }
+}
+
+
+
 static void
 terminal_window_action_prev_tab (GtkAction       *action,
                                  TerminalWindow  *window)
@@ -1541,27 +1598,31 @@ terminal_window_action_goto_tab (GtkRadioAction *action,
 
 
 
+static void
+title_dialog_close (GtkWidget      *dialog,
+                    TerminalWindow *window)
+{
+  terminal_return_if_fail (window->title_dialog == dialog);
+
+  /* need for hiding on focus */
+  if (window->drop_down)
+    terminal_util_activate_window (GTK_WINDOW (window));
+
+  /* close the dialog */
+  window->n_child_windows--;
+  gtk_widget_destroy (dialog);
+  window->title_dialog = NULL;
+}
+
+
+
 static void
 title_dialog_response (GtkWidget      *dialog,
                        gint            response,
                        TerminalWindow *window)
 {
-  /* check if we should open the user manual */
-  if (response == GTK_RESPONSE_HELP)
-    {
-      /* open the "Set Title" paragraph in the "Usage" section */
-      xfce_dialog_show_help (GTK_WINDOW (dialog), "terminal", "usage", NULL);
-    }
-  else
-    {
-      /* need for hiding on focus */
-      if (window->drop_down)
-        terminal_util_activate_window (GTK_WINDOW (window));
-
-      /* close the dialog */
-      window->n_child_windows--;
-      gtk_widget_destroy (dialog);
-    }
+  if (response == GTK_RESPONSE_CLOSE)
+    title_dialog_close (dialog, window);
 }
 
 
@@ -1581,39 +1642,39 @@ terminal_window_action_set_title (GtkAction      *action,
                                   TerminalWindow *window)
 {
   AtkObject *object;
-  GtkWidget *dialog;
+  GtkWidget *button;
   GtkWidget *box;
   GtkWidget *label;
   GtkWidget *entry;
 
-  if (G_LIKELY (window->active != NULL))
+  terminal_return_if_fail (window->active != NULL);
+
+  if (window->title_dialog == NULL)
     {
-      window->n_child_windows++;
+      window->title_dialog = gtk_dialog_new_with_buttons (Q_("Window Title|Set Title"),
+                                                          GTK_WINDOW (window),
+                                                          GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                          NULL,
+                                                          NULL);
 
-      dialog = gtk_dialog_new_with_buttons (Q_("Window Title|Set Title"),
-                                            GTK_WINDOW (window),
-                                            GTK_DIALOG_DESTROY_WITH_PARENT,
-                                            GTK_STOCK_HELP, GTK_RESPONSE_HELP,
-                                            GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-                                            NULL);
-      gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
+      button = xfce_gtk_button_new_mixed ("window-close", _("_Close"));
+      gtk_widget_set_can_default (button, TRUE);
+      gtk_dialog_add_action_widget (GTK_DIALOG (window->title_dialog), button, GTK_RESPONSE_CLOSE);
+      gtk_dialog_set_default_response (GTK_DIALOG (window->title_dialog), GTK_RESPONSE_CLOSE);
 
-      box = gtk_hbox_new (FALSE, 12);
+      box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
       gtk_container_set_border_width (GTK_CONTAINER (box), 6);
-      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), box, TRUE, TRUE, 0);
-      gtk_widget_show (box);
+      gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (window->title_dialog))), box, TRUE, TRUE, 0);
 
       label = gtk_label_new_with_mnemonic (_("_Title:"));
       gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
-      gtk_widget_show (label);
 
       entry = gtk_entry_new ();
       gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
       gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
       gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
-      gtk_entry_set_icon_from_stock (GTK_ENTRY (entry), GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_CLEAR);
+      gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry), GTK_ENTRY_ICON_SECONDARY, "edit-clear");
       g_signal_connect (G_OBJECT (entry), "icon-release", G_CALLBACK (title_dialog_clear), NULL);
-      gtk_widget_show (entry);
 
       /* set Atk description and label relation for the entry */
       object = gtk_widget_get_accessible (entry);
@@ -1623,11 +1684,17 @@ terminal_window_action_set_title (GtkAction      *action,
                               G_OBJECT (entry), "text",
                               G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
 
-      g_signal_connect (G_OBJECT (dialog), "response",
+      g_signal_connect (G_OBJECT (window->title_dialog), "response",
                         G_CALLBACK (title_dialog_response), window);
-
-      gtk_widget_show (dialog);
+      g_signal_connect (G_OBJECT (window->title_dialog), "close",
+                        G_CALLBACK (title_dialog_close), window);
     }
+
+    if (!gtk_widget_get_visible (window->title_dialog))
+      window->n_child_windows++;
+
+    gtk_widget_show_all (window->title_dialog);
+    gtk_window_present (GTK_WINDOW (window->title_dialog));
 }
 
 
@@ -1757,8 +1824,19 @@ static void
 terminal_window_action_contents (GtkAction       *action,
                                  TerminalWindow  *window)
 {
-  /* open the Terminal user manual */
-  xfce_dialog_show_help (GTK_WINDOW (window), "terminal", NULL, NULL);
+  gboolean shortcuts_no_helpkey;
+  gboolean ret;
+
+  g_object_get (G_OBJECT (window->preferences),
+                "shortcuts-no-helpkey", &shortcuts_no_helpkey,
+                NULL);
+
+  if (!shortcuts_no_helpkey)
+    /* open the Terminal user manual */
+    xfce_dialog_show_help (GTK_WINDOW (window), "terminal", NULL, NULL);
+  else
+    /* propagate to the terminal */
+    g_signal_emit_by_name (G_OBJECT (window->active), "key-press-event", gtk_get_current_event (), &ret);
 }
 
 
@@ -1773,6 +1851,25 @@ terminal_window_action_about (GtkAction      *action,
 
 
 
+static void
+terminal_window_zoom_update_screens (TerminalWindow *window)
+{
+  gint            npages, n;
+  TerminalScreen *screen;
+
+  terminal_return_if_fail (GTK_IS_NOTEBOOK (window->notebook));
+
+  /* walk the tabs */
+  npages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (window->notebook));
+  for (n = 0; n < npages; n++)
+    {
+      screen = TERMINAL_SCREEN (gtk_notebook_get_nth_page (GTK_NOTEBOOK (window->notebook), n));
+      terminal_screen_update_font (screen);
+    }
+}
+
+
+
 /**
  * terminal_window_new:
  * @fullscreen: Whether to set the window to fullscreen.
@@ -1855,12 +1952,17 @@ terminal_window_add (TerminalWindow *window,
   label = terminal_screen_get_tab_label (screen);
 
   page = gtk_notebook_append_page (GTK_NOTEBOOK (window->notebook), GTK_WIDGET (screen), label);
-  gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (window->notebook), GTK_WIDGET (screen), TRUE, TRUE, GTK_PACK_START);
+  gtk_container_child_set (GTK_CONTAINER (window->notebook), GTK_WIDGET (screen), "tab-expand", TRUE, NULL);
+  gtk_container_child_set (GTK_CONTAINER (window->notebook), GTK_WIDGET (screen), "tab-fill", TRUE, NULL);
 
   /* allow tab sorting and dnd */
   gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (window->notebook), GTK_WIDGET (screen), TRUE);
   gtk_notebook_set_tab_detachable (GTK_NOTEBOOK (window->notebook), GTK_WIDGET (screen), TRUE);
 
+  /* update screen font from window */
+  if (window->font || window->zoom != TERMINAL_ZOOM_LEVEL_DEFAULT)
+    terminal_screen_update_font (screen);
+
   /* show the terminal screen */
   gtk_widget_realize (GTK_WIDGET (screen));
   gtk_widget_show (GTK_WIDGET (screen));
@@ -1986,6 +2088,11 @@ terminal_window_get_restart_command (TerminalWindow *window)
   else
     result = g_slist_prepend (result, g_strdup ("--hide-toolbar"));
 
+  if (window->zoom != TERMINAL_ZOOM_LEVEL_DEFAULT)
+    result = g_slist_prepend (result, g_strdup_printf ("--zoom=%d", window->zoom));
+  if (window->font != NULL)
+    result = g_slist_prepend (result, g_strdup_printf ("--font=%s", window->font));
+
   /* set restart commands of the tabs */
   children = gtk_container_get_children (GTK_CONTAINER (window->notebook));
   for (lp = children; lp != NULL; lp = lp->next)
